@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -116,6 +116,7 @@ class WSCscrForm   :public WSCform{
     static void _scr_mouse_press(WSCbase*);
     static void _scr_mouse_release(WSCbase*);
     static void _scr_mouse_move(WSCbase*);
+    static void _scr_mouse_dclick(WSCbase*);
 
     WSCushort _get_shift_width();
     WSCushort _get_shift_height();
@@ -131,6 +132,8 @@ class WSCscrForm   :public WSCform{
     WSCbool _get_vbar_visible();
     void _set_no_scroll(WSCbool fl);
     virtual void _adjust_no_scroll();
+static void      (*_ext_draw_proc)(void*);
+static WSCstring _policy_name;
 
   public:
     WSCscrForm(WSCbase*,char*);
@@ -151,6 +154,7 @@ static char* getUseScrBarClass();
     virtual void onScrMousePress(WSCpoint*);
     virtual void onScrMouseRelease(WSCpoint*);
     virtual void onScrMouseMove(WSCpoint*);
+    virtual void onScrMouseDoubleClick(WSCpoint*);
 
     virtual long execEventProc(long ev,void*);
     virtual long clear();
@@ -159,6 +163,7 @@ static char* getUseScrBarClass();
     virtual long cdraw();
     virtual long setPropertyEditMode(WSCbool );
     virtual void  _exec_child_adjust_for_anchors();
+    virtual long getObjectType();
 
     long getVisibleWidth();
     long getVisibleHeight();
@@ -166,4 +171,31 @@ static char* getUseScrBarClass();
     WSMFguiClassDef(WSCscrForm,WSCform);
     WSMFversionDef(WSCscrForm);
 };
+
+struct WSCscrForm_draw_struct{
+  WSCscrForm* _this;
+  WSDdev* dev;
+  short x;
+  short y;
+  WSCushort width;
+  WSCushort height;
+  WSCushort shadow_thickness;
+  short fore_color;
+  short back_color;
+  short top_shadow_color;
+  short bottom_shadow_color;
+  short work_back_color;
+  short back_pixmap;
+  long shadow_type;
+  short gradation;
+  short gradation_margin;
+  WSCushort bar_thickness;
+  WSCuchar  margin_width;
+  WSCuchar  padding;
+  WSCbool   frame;
+  WSCbool   vbar_visible;
+  WSCbool   hbar_visible;
+  
+};
+
 #endif
