@@ -309,7 +309,7 @@ typedef void (*WSCeventProc)(WSCbase*);
 
 #else //TE
 extern "C" {
-extern char*  getcwd(char*,int);
+extern char*  getcwd(char*,unsigned int);
 extern int  chdir(const char*);
 //struct timeval{
 //    int tv_sec;        /* Seconds.  */
@@ -384,6 +384,7 @@ extern int gettimeofday(struct timeval *__tv,
 #define  WSEV_SELECTION_CH        (long)50
 #define  WSEV_SCR_MOUSE_DOUBLE_CLICK  (long)51
 #define  WSEV_ITEM_DOUBLE_CLICKED    (long)52
+#define  WSEV_THREAD_STARTED      (long)53
 
 #define  WSEV_NONE_BIT                0x00000000
 #define  WSEV_INITIALIZE_BIT          0x00000001
@@ -463,6 +464,10 @@ struct WSCrect {
     height = r->height;
   }
 };
+struct WSCrectList {
+  WSCrect rect;
+  WSCrectList* next;
+};
 class WSCstring;
 extern WSDLEX void  WSGFtrace(WSCstring);
 extern WSDLEX char* WSGFstrdup(const char*);
