@@ -130,7 +130,7 @@
   WSGIappLocaleSet()->setLocaleString("SJIS","R3:WSSTR Compiler flags","ʃRpCtO");
   WSGIappLocaleSet()->setLocaleString("SJIS","S3:WSSTR Compile mode","RpC@");
   WSGIappLocaleSet()->setLocaleString("SJIS","T3:WSSTR  Debug mode"," fobO[h");
-  WSGIappLocaleSet()->setLocaleString("SJIS","U3:WSSTR Class library","NXCu");
+  WSGIappLocaleSet()->setLocaleString("SJIS","U3:WSSTR Application type","rh^Cv");
   WSGIappLocaleSet()->setLocaleString("SJIS","V3:WSSTR  Generate class library"," NXCuɂ");
   WSGIappLocaleSet()->setLocaleString("SJIS","W3:WSSTR Compiler","RpC");
   WSGIappLocaleSet()->setLocaleString("SJIS","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode","ʏ탂[h,fobO[h,ʏX^eBbN[h,fobOX^eBbN[h");
