@@ -324,6 +324,17 @@ static void  _popup_after_secs_(WSCuchar,void* data){
   x += 15;
   y += 15;
 
+  short cx,cy;
+  WSGIappMouse()->getMousePosition(&cx,&cy,client);
+  short w = client->getProperty(WSNwidth);
+  short h = client->getProperty(WSNheight);
+  if (cx < 0 || w < cx ){
+    return;
+  }
+  if (cy < 0 || h < cy ){
+    return;
+  }
+
   char* str = (char*)client->getUserData("WSBH-TITLE");
   if (str == NULL){
     str = "No Help";
