@@ -256,3 +256,11 @@ WSCstring WSCdbDrvRecord::getErrorMessage()
 	TRACE_U(("WSCdbDrvRecord::getErrorMessage end"));
 	return "";
 }
+long WSCdbDrvRecord::isNull(long index)
+{
+	return 0;
+}
+long WSCdbDrvRecord::isNull(const char* name)
+{
+	return 0;
+}
