@@ -192,4 +192,6 @@ void WSCmessageDialog::onGuiPolicyChange(long policy){
   }
 }
 
-
+char* WSCmessageDialog::getDefaultProperty(){
+  return WSNlabelString;
+}
