@@ -430,20 +430,20 @@ long WSCvradio::draw(){
     }else{
       dev->setHatchPattern((char)0);
       dev->setForeColor(_bs_color); 
-      dev->drawFillArc(xpos,ypos,isize,isize,60*64,180*64,WS_ARC_PIE);
+      dev->drawFillArc(xpos,ypos,isize,isize,60*64,180*64,WS_PI);
       dev->setForeColor(_ts_color); 
-      dev->drawFillArc(xpos,ypos,isize,isize,240*64,180*64,WS_ARC_PIE);
+      dev->drawFillArc(xpos,ypos,isize,isize,240*64,180*64,WS_PI);
       dev->setForeColor(_indicator_color); 
       dev->drawFillArc(xpos+_shadow_thick,ypos+_shadow_thick,
                        isize - _shadow_thick*2,
-                       isize - _shadow_thick*2,0*64,360*64,WS_ARC_PIE);
+                       isize - _shadow_thick*2,0*64,360*64,WS_PI);
 
       if (_press_status == True ){
         dev->setForeColor(_select_color);
         dev->drawFillArc(xpos+_shadow_thick*2,
                          ypos+_shadow_thick*2,
                          isize - _shadow_thick*4,
-                         isize - _shadow_thick*4,0*64,360*64,WS_ARC_PIE);
+                         isize - _shadow_thick*4,0*64,360*64,WS_PI);
       }
     }
   }
