@@ -55,6 +55,7 @@ WSCvlineGraph::WSCvlineGraph(WSCbase* base, char* objname):
   _data_source_name = WSGFstrdup("");
   _data_source_type = WS_DATA_SOURCE_NONE;
   _need_ds_update = False;
+  _bg_blink_color = WS_DF_NWBACKCOLOR;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCvlineGraph);
@@ -77,6 +78,7 @@ WSCvlineGraph::WSCvlineGraph(WSCbase* base, char* objname):
     WSMFpropertyCreate(WSNdataSource, WSCuchar, _data_source_type,WSSdataSource );
       WSMFpropertySetSelection(WSRdataSource,WSRdataSourceD);
     WSMFpropertyCreate(WSNdataSourceName, char*, _data_source_name,WSSdataSourceName );
+    WSMFpropertyCreate(WSNbackBlinkColor, short,   _bg_blink_color,WSSbackBlinkColor);
 
   WSMFpropertyCreateEnd
 
@@ -93,6 +95,8 @@ WSMFproperty( WSCvlineGraph, WSNhatchPattern, WSCuchar,   _hatch_pattern,0    );
 WSMFproperty( WSCvlineGraph, WSNhatchColor,  short,  _hatch_color,WS_DF_NWBACKCOLOR);
 WSMFproperty( WSCvlineGraph, WSNdataSource,  WSCuchar, _data_source_type,WS_DATA_SOURCE_NONE);
 WSMFproperty( WSCvlineGraph, WSNdataSourceName,  char*, _data_source_name,WSGFstrdup(""));
+WSMFproperty( WSCvlineGraph, WSNbackBlinkColor,    short,        _bg_blink_color, WS_DF_NWBACKCOLOR);
+
 void WSCvlineGraph::_set_ds_instance(){
   WSCbase* serv = WSGIappObjectList()->searchInstance("WSCbase", _data_source_name);
   if (serv != NULL){
@@ -164,6 +168,8 @@ void WSCvlineGraph::setWorkWSNhatchPattern(WSCuchar){ }
 void WSCvlineGraph::getWorkWSNhatchPattern(WSCuchar*){ }
 void WSCvlineGraph::setWorkWSNhatchColor(short){ }
 void WSCvlineGraph::getWorkWSNhatchColor(short*){ }
+void WSCvlineGraph::setWorkWSNbackBlinkColor(short){ }
+void WSCvlineGraph::getWorkWSNbackBlinkColor(short*){ }
 
 WSCvlineGraph::~WSCvlineGraph() {
   if (_values != NULL){
@@ -195,6 +201,14 @@ long WSCvlineGraph::draw(){
   if (absolute == False && dev->isExposed(x, y, w, h) == False){
     return WS_NO_ERR;
   }
+  WSCbool blinkfore = WSGIappBlink()->getBlinkFore(_blink_rate);
+
+  if (_bl_fl == True &&
+      blinkfore == True &&
+      _tw_fl == False) {
+    return WS_NO_ERR;
+  }
+
   WSCbase::update();
   setAbsoluteDraw(False);
   long err = dev->beginDraw(x, y, w, h, absolute);
@@ -210,9 +224,9 @@ long WSCvlineGraph::draw(){
   if (_num < 2){
     _num = 2;
   }
-  WSCushort interval = w/(_num-1);
+  double interval = (double)w/(_num-1);
   if (_orientation != WS_VERTICAL){
-    interval = h/(_num-1);
+    interval = (double)h/(_num-1);
   }
 
   WSCstring vlist(_values);
@@ -221,9 +235,28 @@ long WSCvlineGraph::draw(){
   if (lnum < _num){
     num = lnum;
   }
+  WSCbool not_draw = False;
   dev->setLineWidth(0);
   dev->setHatchPattern(_hatch_pattern);
-  dev->setForeColor(_hatch_color);
+//  dev->setForeColor(_hatch_color);
+  if (_bl_fl != False){
+    if (_tw_fl == False){
+      if (blinkfore == False){
+        dev->setForeColor(_hatch_color);
+      }else{
+        not_draw = True;
+      }
+    }else{
+      if (blinkfore == False){
+        dev->setForeColor(_hatch_color);
+      }else{
+        dev->setForeColor(_bg_blink_color);
+      }
+    }
+  }else{
+    dev->setForeColor(_hatch_color);
+  }
+
   WSCpoint* points = NULL;
   if (_stippled == False){
     points = new WSCpoint[num];
@@ -234,7 +267,7 @@ long WSCvlineGraph::draw(){
     dev->endDraw();
     return WS_ERR;
   }
-  if (_stippled != False){
+  if (_stippled != False && not_draw == False){
     if (_orientation == WS_VERTICAL){
       long na_pt = -1;
       for(i=0; i<num; i++){
@@ -245,14 +278,14 @@ long WSCvlineGraph::draw(){
             continue;
           }
           if (_rev == False){
-            points[i].x = interval*(i-1);
+            points[i].x = (short)(interval*(i-1));
             points[i].y = h -_lw/2;
-            points[i+1].x = interval*(na_pt+1);
+            points[i+1].x = (short)(interval*(na_pt+1));
             points[i+1].y = h -_lw/2;
           }else{
-            points[i].x = interval*(i-1);
+            points[i].x = (short)(interval*(i-1));
             points[i].y = _lw/2;
-            points[i+1].x = interval*(na_pt+1);
+            points[i+1].x = (short)(interval*(na_pt+1));
             points[i+1].y = _lw/2;
           }
           dev->drawFillPoly(&points[na_pt+1],(i-(na_pt+1))+2);
@@ -260,22 +293,22 @@ long WSCvlineGraph::draw(){
         }
         long val = atoi(tmp);
         if (_rev == False){
-          points[i].x = interval*i;
+          points[i].x = (short)(interval*i);
           points[i].y =  h - h*val/100 +_lw/2;
         }else{
-          points[i].x = interval*i;
+          points[i].x = (short)(interval*i);
           points[i].y = h*val/100 +_lw/2;
         }
       }
       if (_rev == False){
-        points[i].x = interval*i;
+        points[i].x = (short)(interval*i);
         points[i].y = h -_lw/2;
-        points[i+1].x = interval*(na_pt+1);
+        points[i+1].x = (short)(interval*(na_pt+1));
         points[i+1].y = h -_lw/2;
       }else{
-        points[i].x = interval*i;
+        points[i].x = (short)(interval*i);
         points[i].y = _lw/2;
-        points[i+1].x = interval*(na_pt+1);
+        points[i+1].x = (short)(interval*(na_pt+1));
         points[i+1].y = _lw/2;
       }
       dev->drawFillPoly(&points[na_pt+1],(i-(na_pt+1))+2);
@@ -290,14 +323,14 @@ long WSCvlineGraph::draw(){
           }
           if (_rev == False){
             points[i].x = w - _lw/2;
-            points[i].y = interval*(i-1);
+            points[i].y = (short)(interval*(i-1));
             points[i+1].x = w - _lw/2;
-            points[i+1].y = interval*(na_pt+1);
+            points[i+1].y = (short)(interval*(na_pt+1));
           }else{
             points[i].x = _lw/2;
-            points[i].y = interval*(i-1);
+            points[i].y = (short)(interval*(i-1));
             points[i].x = _lw/2;
-            points[i+1].y = interval*(na_pt+1);
+            points[i+1].y = (short)(interval*(na_pt+1));
           }
           dev->drawFillPoly(&points[na_pt+1],(i-(na_pt+1))+2);
           na_pt = i;
@@ -305,29 +338,49 @@ long WSCvlineGraph::draw(){
         long val = atoi(tmp);
         if (_rev == False){
           points[i].x = w - w*val/100 + _lw/2;
-          points[i].y = interval*i;
+          points[i].y = (short)(interval*i);
         }else{
           points[i].x = w*val/100 +_lw/2;
-          points[i].y = interval*i;
+          points[i].y = (short)(interval*i);
         }
       }
       if (_rev == False){
         points[i].x = w - _lw/2;
-        points[i].y = interval*i;
+        points[i].y = (short)(interval*i);
         points[i+1].x = w - _lw/2;
-        points[i+1].y = interval*(na_pt+1);
+        points[i+1].y = (short)(interval*(na_pt+1));
       }else{
         points[i].x = _lw/2;
-        points[i].y = interval*i;
+        points[i].y = (short)(interval*i);
         points[i].x = _lw/2;
-        points[i+1].y = interval*(na_pt+1);
+        points[i+1].y = (short)(interval*(na_pt+1));
       }
       dev->drawFillPoly(&points[na_pt+1],(i-(na_pt+1))+2);
     }
   }
   dev->setLineWidth(_lw);
   dev->setLineDashType(_lt);
-  dev->setForeColor(_fore_color);
+//  dev->setForeColor(_fore_color);
+
+  not_draw = False;
+  if (_bl_fl != False){
+    if (_tw_fl == False){
+      if (blinkfore == False){
+        dev->setForeColor(_fore_color);
+      }else{
+        not_draw = True;
+      }
+    }else{
+      if (blinkfore == False){
+        dev->setForeColor(_fore_color);
+      }else{
+        dev->setForeColor(_blink_color);
+      }
+    }
+  }else{
+    dev->setForeColor(_fore_color);
+  }
+
   dev->setHatchPattern(0);
 
   if (_orientation == WS_VERTICAL){
@@ -349,14 +402,16 @@ long WSCvlineGraph::draw(){
       }
       long val = atoi(tmp);
       if (_rev == False){
-        points[i].x = interval*i;
+        points[i].x = (short)(interval*i);
         points[i].y =  h - h*val/100 +_lw/2;
       }else{
-        points[i].x = interval*i;
+        points[i].x = (short)(interval*i);
         points[i].y = h*val/100 +_lw/2;
       }
     }
-    dev->drawLines(&points[na_pt+1],i-(na_pt+1));
+    if (not_draw == False){
+      dev->drawLines(&points[na_pt+1],i-(na_pt+1));
+    }
   }else{
     long na_pt = -1;
     for(i=0; i<num; i++){
@@ -377,13 +432,15 @@ long WSCvlineGraph::draw(){
       long val = atoi(tmp);
       if (_rev == False){
         points[i].x = w - w*val/100 + _lw/2;
-        points[i].y = interval*i;
+        points[i].y = (short)(interval*i);
       }else{
         points[i].x = w*val/100 + _lw/2;
-        points[i].y = interval*i;
+        points[i].y = (short)(interval*i);
       }
     }
-    dev->drawLines(&points[na_pt+1],i-(na_pt+1));
+    if (not_draw == False){
+      dev->drawLines(&points[na_pt+1],i-(na_pt+1));
+    }
   }
   delete points;
   dev->endDraw();
@@ -413,3 +470,6 @@ void WSCvlineGraph::update(){
   }
   WSCnwbase::update();
 }
+char* WSCvlineGraph::getDefaultProperty(){
+  return WSNvalue;
+}
