@@ -479,11 +479,11 @@ void _destroy_evt(W window,WEVENT* ev){
   WSDbtronFormDev* dev;
   W ret = wget_dat(window,(W*)&dev);
   if (ret != 0){
-dbprintf("_btn_press(W=0x%x) is invalid.\n",window);
+dbprintf("_destroy_evt(W=0x%x) is invalid.\n",window);
     return;
   }
   if (dev == NULL){
-dbprintf("_btn_press(W=0x%x) has no instance.\n",window);
+dbprintf("_destroy_evt(W=0x%x) has no instance.\n",window);
     return;
   }
   dev->_evh_destroy(ev);
@@ -744,12 +744,12 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x2 done\n",__FILE__,__LINE__);
   }else if (msg->msg_type == MS_TYPE3){ 
 #ifndef TE
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x2 start\n",__FILE__,__LINE__);
-printf("recvmsg.. type=3\n");
+//printf("recvmsg.. type=3\n");
     memcpy(&data,msg->msg_body.ANYMSG.msg_str,sizeof(data));
     WSDbtronExecute* obj = data.obj;
-printf("this=0x%x id=%d,%d sizeof(data)=%D\n",obj,obj->getId(),data.id,sizeof(data));
+//printf("this=0x%x id=%d,%d sizeof(data)=%D\n",obj,obj->getId(),data.id,sizeof(data));
     if (obj->getId() == data.id){
-printf("this=0x%x\n",obj);
+//printf("this=0x%x\n",obj);
       WSDbtronExecute::_callback(obj);
     }
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x2 done\n",__FILE__,__LINE__);
@@ -796,8 +796,9 @@ long WSDbtronAppDev::_dispatchEvent(WSCbool wait){
   W ret = wget_evt(&ev,w);
   W ty;
 
-//if (ret != EV_NULL) dbprintf("EV=0x%x\n",ret);
+//if (ret != EV_NULL){ dbprintf("EV=0x%x\n",ret);
 //printf("EV=%d.. key=0x%x code=0x%x status=0x%x\n",ret,ev.s.cmd,ev.s.wid,ev.s.stat);
+//}
 //printf("mask=0x%x\n",chg_emk(-1));
 //printf("EV=%d.. key=0x%x code=0x%x status=0x%x mask=0x%x\n",ret,ev.s.cmd,ev.s.wid,ev.s.stat,chg_emk(-1));
 
@@ -815,6 +816,10 @@ dbprintf("EV_NOMSG.. wnd=0x%x\n",ev.s.wid);
     case EV_MSG:{
 dbprintf("EV_MSG.. wnd=0x%x\n",ev.s.wid);
       recvmsg();
+      WSDbtronFormDev::executeUpdate();
+      WSGIappObjectList()->execInitialize();
+      WSGIappObjectList()->execUpdate();
+
       return WS_NO_ERR;
     }
     case EV_MENU:{
@@ -874,7 +879,21 @@ dbprintf("EV_BUTDWN.. stat=0x%x\n",ev.s.stat);
       switch(ev.s.cmd){
         case W_WORK:{
 dbprintf("W_WORK.. wnd=0x%x\n",ev.s.wid);
-          _btn_press(ev.s.wid,&ev);
+          WSDbtronFormDev* mouse_target = _mouse_grab_target;
+          if (mouse_target != NULL &&
+              WSDbtronFormDev::isValidInstance(mouse_target) == False){
+            mouse_target = NULL;
+//printf("EV_BUTDOWN.. mouse_target=NULL! wnd=0x%x\n",mouse_target);
+          }
+          if (mouse_target != NULL){
+//printf("EV_BUTDOWN.. mouse_target=0x%x client=0x%x\n",mouse_target,mouse_target->getAttachedClient());
+//if (mouse_target->getAttachedClient() != NULL){
+//printf("EV_BUTDOWN.. %s\n",mouse_target->getAttachedClient()->getInstanceName());
+//}
+            mouse_target->_evh_btn_press(&ev);
+          }else{
+            _btn_press(ev.s.wid,&ev);
+          }
           break;
         }
         case W_LTHD:
