@@ -433,6 +433,9 @@ void WSDbtronFormDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,lon
     if (getParentGID() != NULL){
       gcop_bmp(getParentGID(),&from,getParentGID(),&to,NULL,G_STORE);
     }
+    if (getParentMemGID() != NULL){
+      gcop_bmp(getParentMemGID(),&from,getParentMemGID(),&to,NULL,G_STORE);
+    }
   }
 }
 
