@@ -49,6 +49,7 @@ extern "C" {
 #include <btron/WSDbtronColor.h>
 
 long WSGFbtronBufToPixmap(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes);
+long WSGFbtronBufToPixmapG(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes);
 long WSGFbtronBmpReadFpToPixmap(FILE* ifile,W* pixmap,long* w,long* h);
 long WSGFbtronBmpReadFileToPixmap(char* filename,W* pixmap,long* w,long* h);
 long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h);
@@ -577,6 +578,84 @@ long WSGFbtronBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight
   }
   return 0;
 }
+long WSGFbtronBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheight,long* w,long* h,long row_bytes){
+
+  long stretched = 0;
+  long vwidth = *w;
+  long vheight = *h;
+  if (vwidth == 0 || vheight == 0){
+    vwidth = rwidth;
+    vheight = rheight;
+  }else{
+    if ((long)((double)rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)((double)vheight * rwidth/rheight);
+    }else{
+      vheight = (long)((double)vwidth * rheight/rwidth);
+    }
+    stretched = 1;
+  }
+  *w = vwidth;
+  *h = vheight;
+
+  *mem_gid = WSGIbtronAppDev()->getNewMemGid(vwidth,vheight);
+//printf(">>>> mem_gid=0x%x  w,h=%d,%d\n",*mem_gid,vwidth,vheight);
+  if (*mem_gid == 0){
+    return -1;
+  }
+  long i,j;
+  long ptr=0;
+  unsigned char r,g,b;
+#if 0 //need not..
+  DEV_SPEC* dev_spec = WSGIbtronAppDev()->getDevSpec();
+  WSCulong red_shift = 0;
+  WSCulong red_bits = 0;
+  WSCulong green_shift = 0;
+  WSCulong green_bits = 0;
+  WSCulong blue_shift = 0;
+  WSCulong blue_bits = 0;
+  WSCbool speed_up = False;
+  if (dev_spec != NULL && ((dev_spec->attr & 0x3) == 1) &&
+      ((dev_spec->attr & 0x4) == 0)){
+    red_shift = (dev_spec->color[0] & 0xff00) >> 8;
+    green_shift = (dev_spec->color[1] & 0xff00) >> 8;
+    blue_shift = (dev_spec->color[2] & 0xff00) >> 8;
+    red_bits = (dev_spec->color[0] & 0xff);
+    green_bits = (dev_spec->color[1] & 0xff);
+    blue_bits = (dev_spec->color[2] & 0xff);
+    speed_up = True;
+  }
+printf("speed_up=%d dev_spec->attr=0x%x 0x%x 0x%x\n",speed_up,dev_spec->attr,dev_spec->attr & 0x3, dev_spec->attr & 0x4);
+#endif
+  for(i=0; i<vheight; i++){
+    for(j=0; j<vwidth; j++){
+      if (stretched != 0){
+        ptr = (long)((double)i*rheight/vheight)*rwidth
+             + (long)((double)j*rwidth/vwidth);
+      }
+      r = buf[ptr];
+      g = buf[ptr];
+      b = buf[ptr++];
+      unsigned long val = 0x10000000;
+      PIXVAL pixval = 0;
+//      if (speed_up == False){
+        val |= r << 16;
+        val |= g << 8;
+        val |= b;
+        gcnv_col(*mem_gid,val,&pixval);
+//      }else{
+//        pixval |=  (r >> (8 - red_bits)) << red_shift;
+//        pixval |=  (g >> (8 - green_bits)) << green_shift;
+//        pixval |=  (b >> (8 - blue_bits)) << blue_shift;
+//      }
+      PNT pt;
+      pt.x = j;
+      pt.y = i;
+      gdra_pnt(*mem_gid,pt,pixval,G_STORE);
+    }
+    ptr = row_bytes*i;
+  }
+  return 0;
+}
 long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth,long rheight,long* w,long* h,long row_bytes){
 
   long stretched = 0;
@@ -620,7 +699,7 @@ long WSGFbtronBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
       val |= r << 16;
       val |= g << 8;
       val |= b;
-      PIXVAL pixval;
+      PIXVAL pixval = 0;
       gcnv_col(*mem_gid,val,&pixval);
       PNT pt;
       pt.x = j;
@@ -655,6 +734,8 @@ long WSGFbtronBmpReadFileToPixmap(char* filename,W* pixmap,long* w,long* h){
 void png_cexcept_error(png_structp png_ptr, png_const_charp msg){
    if(png_ptr){
      fprintf(stderr, "png file read error: %s\n", msg);
+     bool*  png_err_flag_ptr = (bool*) png_get_error_ptr(png_ptr);
+     *png_err_flag_ptr = true;
    }
 }
 void png_read_data_fn(png_structp png_ptr,png_bytep data,png_size_t length){
@@ -677,7 +758,9 @@ long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long*
       fclose(ifile);
       return -1;
     }
-    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL,
+    bool png_err_flag = false;
+    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
+      (png_voidp) &png_err_flag,
       (png_error_ptr)png_cexcept_error, (png_error_ptr)NULL);
     if (png_ptr == NULL){
       fclose(ifile);
@@ -695,6 +778,13 @@ long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long*
     png_set_read_fn(png_ptr,(png_voidp)ifile,png_read_data_fn);
     png_set_sig_bytes(png_ptr, 8);
     png_read_info(png_ptr, info_ptr);
+
+    if (png_err_flag != false)  {
+      png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
+      fclose(ifile);
+      return -1;
+    }
+
     png_uint_32 png_width;
     png_uint_32 png_height;
     int png_depth;
@@ -704,6 +794,54 @@ long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long*
     if (png_depth == 16){
       png_set_strip_16(png_ptr);
     }
+#if 1 //replace new logic
+    if (png_depth < 8)
+        png_set_packing(png_ptr);
+    //  Transformations
+    //  from readpng2_info_callback() in contrib/gregbook/readpng2.c
+    if (png_color_type == PNG_COLOR_TYPE_PALETTE)
+        png_set_expand(png_ptr);
+    if (png_color_type == PNG_COLOR_TYPE_GRAY && png_depth < 8)
+        png_set_expand(png_ptr);
+    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
+        png_set_expand(png_ptr);
+    if (png_color_type == PNG_COLOR_TYPE_GRAY ||
+        png_color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
+        png_set_gray_to_rgb(png_ptr);
+
+    // Ignore background color with transparent and alpha
+    // png_get_bKGD(), png_set_background()
+
+    // Gamma conversion
+    //  example.c 
+    int intent;
+    //screen_gamma = 2.2; // A good guess for a PC monitor
+    //                    //   in a bright office or a dim room
+    //screen_gamma = 2.0; // A good guess for a PC monitor
+    //                    //   in a dark room
+    const double screen_gamma = 2.2;
+    if (png_get_sRGB(png_ptr, info_ptr, &intent))
+       png_set_gamma(png_ptr, screen_gamma, 0.45455);
+    else
+    {
+       double image_gamma;
+       if (png_get_gAMA(png_ptr, info_ptr, &image_gamma))
+          png_set_gamma(png_ptr, screen_gamma, image_gamma);
+       else
+          png_set_gamma(png_ptr, screen_gamma, 0.45455);
+    }
+
+    // after the transformations have been registered update info_ptr data
+    png_read_update_info(png_ptr, info_ptr);
+
+    // need this code?
+    // get again width, height and the new bit-depth and color-type
+    png_get_IHDR(png_ptr, info_ptr, &png_width, &png_height, &png_depth,
+            &png_color_type, NULL, NULL, NULL);
+
+    png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
+    png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
+#else
     png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
     png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
     if (png_depth < 8){
@@ -762,6 +900,7 @@ dbprintf("IMG> PNG_COLOR_TYPE_PALETTE\n");
       png_row_bytes = png_width * 3;
       png_channels = 3;
     }
+#endif
 
     long rwidth = png_width;
     long rheight = png_height;
@@ -785,6 +924,7 @@ fprintf(stderr,"png read error: out of memory..\n");
 fprintf(stderr,"png read error: out of memory..\n");
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
       fclose(ifile);
+      delete buf;
       return -1;
     }
     int i;
@@ -954,6 +1094,11 @@ long JpegReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long
     if (components == 3){
       *mem_gid2 = 0;
       WSGFbtronBufToPixmap(buf, mem_gid,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE));
+    }else
+    if (components == 1){
+      *mem_gid2 = 0;
+      WSGFbtronBufToPixmapG(buf, mem_gid,rwidth,rheight,w,h,components*rwidth*sizeof(JSAMPLE));
+
     }else{
       delete buf;
       jpeg_finish_decompress(&jinf);
