@@ -505,7 +505,48 @@ long WSDbtronScrFrameDev::createPixmap(){
   exposeExecute(0,0,w,h);
   return WS_NO_ERR;
 #endif
-  return WSDbtronFormDev::createPixmap();
+
+//  return WSDbtronFormDev::createPixmap();
+
+//dbprintf("CC WSDbtronFormDev::createPixmap() %s\n",getAttachedClient()->getInstanceName());
+  short x,y;
+  WSCushort w = _work_width;
+  WSCushort h = _work_height;
+  
+  if (w <1){
+    w = 1;
+  }
+  if (h <1){
+    h = 1;
+  }
+
+//  _mem_gid = WSGIbtronAppDev()->getNewMemGid(w,h);
+  if (getParentMemGID() == 0){
+//printf("WSDbtronFormDev::createPixmap() error return %s\n",w,h,getAttachedClient()->getInstanceName());
+    return WS_ERR;
+  }
+  WSDwindowDev::createPixmap();
+//printf("CC WSDbtronFormDev::createPixmap() wh=%d,%d %s\n",w,h,getAttachedClient()->getInstanceName());
+//dbprintf("WSDbtronFormDev::createPixmap() wh=%d,%d %s\n",w,h,getAttachedClient()->getInstanceName());
+  _setup_pixmap(0,0,w,h,_bg,_bg_pix);
+  exposeExecute(0,0,w,h);
+#ifdef F_MEMGID
+  WSClistData children = getChildDevList();
+  long num = children.getNum();
+  long i;
+  for(i=0; i<num; i++){
+    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    if (WSDbtronFormDev::isValidInstance(child) == False){
+      continue;
+    }
+    if (child->getParentVisible() != False){
+      child->createPixmap();
+    }
+  }
+#endif
+
+  return WS_NO_ERR;
+
 }
 
 void WSDbtronScrFrameDev::getDispAddr(short* ox,short* oy){
