@@ -43,7 +43,7 @@
 
 #ifdef TE
 extern "C" {
-char* getcwd(char* buf,int len){
+char* getcwd(char* buf,unsigned int len){
   strncpy(buf,"/SYS",len);
   return buf;
 }
