@@ -1091,3 +1091,14 @@ WSCstring WSCdbMysqlRecord::getErrorMessage()
   ret.setString(buf2);
   return ret;
 }
+long WSCdbMysqlRecord::isNull(long index)
+{
+	if(index < 0 || index >= (_Cols-_rowiduse)) {
+		return -1;
+	}
+	return (*((char*)_fd[index].buff) == '\0' ? True : False);
+}
+long WSCdbMysqlRecord::isNull(const char* name)
+{
+	return isNull(getColIndex(name));
+}
