@@ -630,7 +630,7 @@ long WSDnxAppDev::_dispatchEvent(WSCbool wait){
       case WSEV_NX_TIMER:{
         _timer_send_type* data = (_timer_send_type*)ev.data;
         WSDnxPrivateTimer* timer = data->timer;
-        WSCulong tid = data->tid;
+        pthread_t tid = data->tid;
         if (WSDnxPrivateTimer::isValid(timer)){
           timer->_timer_callback(tid);
         }
