@@ -1692,7 +1692,8 @@ WSCbool WSDnxFormDev::getParentVisible(){
 void WSDnxFormDev::getParentAbstPos(short* x,short* y){
   WSDnxFormDev* parent = getParentAreaDev();
   if (parent != NULL){
-    return parent->getParentAbstPos(x,y);
+    parent->getParentAbstPos(x,y);
+    return;
   }
   *x = _w_x;
   *y = _w_y;
