@@ -366,3 +366,19 @@ WSCstring WSCdbOdbcRecord::getErrorMessage(){
   }
   return ret;
 }
+long WSCdbOdbcRecord::isNull(long index)
+{
+	long ret = WS_ERR;
+	if(_rs) {
+		ret = _rs->isNull(index);
+	}
+	return ret;
+}
+long WSCdbOdbcRecord::isNull(const char* name)
+{
+	long ret = WS_ERR;
+	if(_rs) {
+		ret = _rs->isNull(name);
+	}
+	return ret;
+}
