@@ -492,6 +492,9 @@ long WSCodbcRecord::moveNext()
 	if(_eof) {
 		return WS_ERR;
 	}
+	for(int i = 0; i < _Cols; i++) {
+		memset(_fd[i].buff, '\0', _fd[i].buffLen);
+	}
 	_access_mode = CODBC_ACCESS_MODE_READ;
 // update 2002/06/05
 //    sqlrc = SQLFetch(_hstmt);
@@ -504,6 +507,9 @@ long WSCodbcRecord::moveNext()
 long WSCodbcRecord::movePrevious()
 {
 	SQLRETURN	sqlrc = SQL_SUCCESS;
+	for(int i = 0; i < _Cols; i++) {
+		memset(_fd[i].buff, '\0', _fd[i].buffLen);
+	}
 	_access_mode = CODBC_ACCESS_MODE_READ;
 	sqlrc = SQLFetchScroll(_hstmt, SQL_FETCH_PRIOR, 1);
 	if(sqlrc == SQL_ROW_SUCCESS) {
@@ -514,6 +520,9 @@ long WSCodbcRecord::movePrevious()
 long WSCodbcRecord::moveFirst()
 {
 	SQLRETURN	sqlrc = SQL_SUCCESS;
+	for(int i = 0; i < _Cols; i++) {
+		memset(_fd[i].buff, '\0', _fd[i].buffLen);
+	}
 	_access_mode = CODBC_ACCESS_MODE_READ;
 	sqlrc = SQLFetchScroll(_hstmt, SQL_FETCH_FIRST, 1);
 	if(sqlrc == SQL_ROW_SUCCESS) {
@@ -527,6 +536,9 @@ long WSCodbcRecord::moveLast()
 	if(_eof) {
 		return WS_ERR;
 	}
+	for(int i = 0; i < _Cols; i++) {
+		memset(_fd[i].buff, '\0', _fd[i].buffLen);
+	}
 	_access_mode = CODBC_ACCESS_MODE_READ;
 	sqlrc = SQLFetchScroll(_hstmt, SQL_FETCH_LAST, 1);
 	return WS_NO_ERR;
@@ -879,3 +891,14 @@ WSCstring WSCodbcRecord::getErrorMessage(){
   ret.setString(buf2);
   return ret;
 }
+long WSCodbcRecord::isNull(long index)
+{
+	if(index < 0 || index >= (_Cols)) {
+		return -1;
+	}
+	return (*((char*)_fd[index].buff) == '\0' ? True : False);
+}
+long WSCodbcRecord::isNull(const char* name)
+{
+	return isNull(getColIndex(name));
+}
