@@ -39,6 +39,8 @@ extern long PngWriteBufToFp(FILE* fp, char* buf,long width,long height);
 void png_cexcept_error(png_structp png_ptr, png_const_charp msg){
    if(png_ptr){
      fprintf(stderr, "png file read error: %s\n", msg);
+     bool*  png_err_flag_ptr = (bool*) png_get_error_ptr(png_ptr);
+     *png_err_flag_ptr = true;
    }
 }
 
@@ -66,7 +68,9 @@ long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pix
       fclose(ifile);
       return -1;
     }
-    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL,
+    bool png_err_flag = false;
+    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
+      (png_voidp) &png_err_flag,
       (png_error_ptr)png_cexcept_error, (png_error_ptr)NULL);
     if (png_ptr == NULL){
       fclose(ifile);
@@ -82,6 +86,13 @@ long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pix
     png_init_io(png_ptr, ifile);
     png_set_sig_bytes(png_ptr, 8);
     png_read_info(png_ptr, info_ptr);
+
+    if (png_err_flag != false)  {
+      png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
+      fclose(ifile);
+      return -1;
+    }
+
     png_uint_32 png_width;
     png_uint_32 png_height;
     int png_depth;
@@ -91,6 +102,54 @@ long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pix
     if (png_depth == 16){
       png_set_strip_16(png_ptr);
     }
+#if 1 //replace new logic
+    if (png_depth < 8)
+        png_set_packing(png_ptr);
+    //  Transformations
+    //  from readpng2_info_callback() in contrib/gregbook/readpng2.c
+    if (png_color_type == PNG_COLOR_TYPE_PALETTE)
+        png_set_expand(png_ptr);
+    if (png_color_type == PNG_COLOR_TYPE_GRAY && png_depth < 8)
+        png_set_expand(png_ptr);
+    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
+        png_set_expand(png_ptr);
+    if (png_color_type == PNG_COLOR_TYPE_GRAY ||
+        png_color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
+        png_set_gray_to_rgb(png_ptr);
+
+    // Ignore background color with transparent and alpha
+    // png_get_bKGD(), png_set_background()
+
+    // Gamma conversion
+    //  example.c 
+    int intent;
+    //screen_gamma = 2.2; // A good guess for a PC monitor
+    //                    //   in a bright office or a dim room
+    //screen_gamma = 2.0; // A good guess for a PC monitor
+    //                    //   in a dark room
+    const double screen_gamma = 2.2;
+    if (png_get_sRGB(png_ptr, info_ptr, &intent))
+       png_set_gamma(png_ptr, screen_gamma, 0.45455);
+    else
+    {
+       double image_gamma;
+       if (png_get_gAMA(png_ptr, info_ptr, &image_gamma))
+          png_set_gamma(png_ptr, screen_gamma, image_gamma);
+       else
+          png_set_gamma(png_ptr, screen_gamma, 0.45455);
+    }
+
+    // after the transformations have been registered update info_ptr data
+    png_read_update_info(png_ptr, info_ptr);
+
+    // need this code?
+    // get again width, height and the new bit-depth and color-type
+    png_get_IHDR(png_ptr, info_ptr, &png_width, &png_height, &png_depth,
+            &png_color_type, NULL, NULL, NULL);
+
+    png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
+    png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
+#else
     png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
     png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
 
@@ -147,6 +206,8 @@ long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pix
       png_row_bytes = png_width * 3;
       png_channels = 3;
     }
+#endif
+
     long rwidth = png_width;
     long rheight = png_height;
     long components = png_channels;
@@ -168,6 +229,7 @@ fprintf(stderr,"png read error: out of memory..\n");
 fprintf(stderr,"png read error: out of memory..\n");
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
       fclose(ifile);
+      delete buf;
       return -1;
     }
     int i;
