@@ -1176,3 +1176,14 @@ WSCstring WSCdbPgsqlRecord::getErrorMessage()
   ret.setString(buf2);
   return ret;
 }
+long WSCdbPgsqlRecord::isNull(long index)
+{
+	if(index < 0 || index >= (_Cols-_oiduse)) {
+		return -1;
+	}
+	return (*((char*)_fd[index+_oiduse].buff) == '\0' ? True : False);
+}
+long WSCdbPgsqlRecord::isNull(const char* name)
+{
+	return isNull(getColIndex(name));
+}
