@@ -8,7 +8,8 @@ typedef unsigned long  WSCulong;
 typedef unsigned char  WSCuchar;
 typedef unsigned int   WSCuint;
 
-#ifdef SWIGRUBY
+/*#ifdef SWIGRUBY*/
+#if 0 /* deleted .. v3.60-2*/
 /* WSCbool to (or from) Ruby's Truth value. */
 %typemap(in) WSCbool {
   if (($input == Qfalse) || ($input == Qnil)) {
@@ -379,7 +380,6 @@ public:
   ~WSCform();
 };
 
-
 class WSCnwbase :public WSCbase{
 public:
   WSCnwbase(WSCbase*,char*);
@@ -707,11 +707,25 @@ public:
   long setLineWidth(short);
   long setLineDashType(char no);
   long setHatchPattern(char no);
+#ifdef SWIGPYTHON
+  %extend {
+    long setLineDashType(short no){ return self->setLineDashType((char)no); }
+    long setHatchPattern(short no){ return self->setHatchPattern((char)no); }
+  }
+#endif
 
   long setRegion(short x,short y,WSCushort w,WSCushort h);
   long drawArc(short x,short y,WSCushort width,WSCushort height,short,short);
   long drawFillArc(short x,short y,WSCushort width,WSCushort height,short _a1,
                    short _a2,char kind);
+#ifdef SWIGPYTHON
+  %extend {
+    long drawFillArc(short x,short y,WSCushort width,WSCushort height,
+                     short _a1, short _a2,short kind) {
+      return self->drawFillArc(x, y, width, height, _a1, _a2, (char)kind);
+    }
+  }
+#endif
   long drawLine(short x1,short y1,short x2,short y2);
   long drawLines(WSCpoint*,long num);
   long drawRect(short x,short y,WSCushort w,WSCushort h);
@@ -734,6 +748,21 @@ public:
 
   long drawFillString(long x,long y,WSCulong w,WSCulong h,char font,char align,
                         char*,long encoding = WS_EN_DEFAULT);
+#ifdef SWIGPYTHON
+  %extend {
+    long drawImage(short x,short y,WSCushort w,WSCushort h,char* img,short align) {
+      return self->drawImage(x,y,w,h,img,(char)align);
+    }
+    long drawString(long x,long y,WSCulong w,WSCulong h,short font,short align,
+                            char* str,long encoding = WS_EN_DEFAULT) {
+      return self->drawString(x,y,w,h,(char)font,(char)align,str,encoding);
+    }
+    long drawFillString(long x,long y,WSCulong w,WSCulong h,short font,short align,
+                        char* str,long encoding = WS_EN_DEFAULT) {
+      return self->drawFillString(x,y,w,h,(char)font,(char)align,str,encoding);
+    }
+  }
+#endif
 };
 
 class WSCvbarGraph :public WSCnwbase{
@@ -854,6 +883,75 @@ public:
   WSCstring getErrorMessage();
 };
 
+/* SQL_CONCURRENCY options */
+#define WSCDB_SQL_CONCUR_READ_ONLY            1
+#define WSCDB_SQL_CONCUR_LOCK                 2
+#define WSCDB_SQL_CONCUR_ROWVER               3
+#define WSCDB_SQL_CONCUR_VALUES               4
+#define WSCDB_SQL_CONCUR_DEFAULT              WSCDB_SQL_CONCUR_READ_ONLY
+
+/* SQL_CURSOR_TYPE options */
+#define WSCDB_SQL_CURSOR_FORWARD_ONLY         0UL
+#define WSCDB_SQL_CURSOR_KEYSET_DRIVEN        1UL
+#define WSCDB_SQL_CURSOR_DYNAMIC              2UL
+#define WSCDB_SQL_CURSOR_STATIC               3UL
+#define WSCDB_SQL_CURSOR_TYPE_DEFAULT         WSCDB_SQL_CURSOR_FORWARD_ONLY
+
+// FIELD TYPE
+#define WSCDB_FIELD_TYPE_CHAR                 1
+#define WSCDB_FIELD_TYPE_NUMERIC              2
+#define WSCDB_FIELD_TYPE_DECIMAL              3
+#define WSCDB_FIELD_TYPE_INTEGER              4
+#define WSCDB_FIELD_TYPE_SMALLINT             5
+#define WSCDB_FIELD_TYPE_FLOAT                6
+#define WSCDB_FIELD_TYPE_REAL                 7
+#define WSCDB_FIELD_TYPE_DOUBLE               8
+#define WSCDB_FIELD_TYPE_DATE                 9
+#define WSCDB_FIELD_TYPE_TIME                 10
+#define WSCDB_FIELD_TYPE_TIMESTAMP            11
+#define WSCDB_FIELD_TYPE_NONE                 99
+
+class WSCdbRecord
+{
+public:
+	//WSCdbRecord(WSCdbDatabase* pDatabase);
+	WSCdbRecord(WSCvdb* inst);
+	virtual ~WSCdbRecord();
+
+	virtual long    open(const char* stmt);
+	virtual long    open(const char* stmt, short eCousor, short eAccess = WSCDB_SQL_CONCUR_LOCK);
+	virtual long    close();
+	virtual WSCbool isEOF();
+	virtual WSCbool isOpen();
+        %apply WSCvariant *OUTPUT { WSCvariant *var };
+	virtual long    getColValue(long index, WSCvariant* var);
+	virtual long    getColValue(const char* name, WSCvariant* var);
+	//virtual long    getColStringValue(long index, char* var);
+	//virtual long    getColStringValue(const char* name, char* var);
+	virtual long    setColValue(long index, WSCvariant var);
+	virtual long    setColValue(const char* name, WSCvariant var);
+	virtual long    edit();
+	virtual long    update();
+	virtual long    addnew();
+	virtual long    deleterow();
+	virtual long    moveNext();
+	virtual long    movePrevious();
+	virtual long    moveFirst();
+	virtual long    moveLast();
+	//virtual long    getColName(long index, char* name);
+	virtual long    getColIndex(const char* name);
+	virtual long    getColType(long index);
+	virtual long    getColType(const char* name);
+	virtual long    getColLength(long index);
+	virtual long    getColLength(const char* name);
+	virtual long    getRowCount();
+	virtual long    getColCount();
+	//virtual void    getErrorMsg(char* szState, char* szMsg);
+
+public:
+	//WSCdbDatabase*	_pDB;
+	//WSCdbDrvRecord*	_rs;
+};
 
 class WSCwindow :public WSCbase{
 public:
@@ -982,11 +1080,60 @@ public:
   ~WSCmenuArea();
 };
 
+class WSCtform   :public WSCform{
+  public:
+    WSCtform(WSCbase*,char*);
+    virtual ~WSCtform();
+};
 
+class WSCfform :public WSCbase{
+  public:
+    WSCfform(WSCbase*,char*);
+    virtual ~WSCfform();
 
+    void execResizeEvent();
+    void setNoRefresh(WSCbool);
+    WSCbool getNoRefresh();
+    void setFloated(WSCbool);
+    WSCbool getFloated();
+    long getPixmapStyle();
+};
 
+class WSCprform   :public WSCform{
+  public:
+    WSCprform(WSCbase*,char*);
+    virtual ~WSCprform();
 
+    virtual long prdraw();
+};
 
+class WSCj3wform   :public WSCform{
+  public:
+    WSCj3wform(WSCbase*,char*);
+    virtual ~WSCj3wform();
+
+    long loadFile(char*);
+};
+#if 0
+class WSCopenglForm   :public WSCform{
+  public:
+
+    WSCopenglForm(WSCbase* base, char* name);
+
+    virtual ~WSCopenglForm();
+
+    //void* getOpenGLContext();
+
+    virtual long beginDraw();
+    virtual long endDraw();
+
+    //void setCreateContextHandler(long(*hd)());
+    //void setUpdateContextHandler(long(*hd)());
+    void makeCurrent();
+    void noMakeCurrent();
+
+};
+#endif
 
 class WSCscrForm :public WSCform{
 public:
@@ -1376,6 +1523,20 @@ public:
   long      setFixedCellAlignemnt(long col, long row, char align);
   char      getFixedCellAlignemnt(long col, long row);
 
+#ifdef SWIGPYTHON
+  %extend {
+    long  setCellAlignemnt(short align) {
+      return self->setCellAlignemnt((char)align);
+    }
+    long  setCellAlignemnt(long col, long row, short align) {
+      return self->setCellAlignemnt(col, row, (char)align);
+    }
+    long  setFixedCellAlignemnt(long col, long row, short align) {
+      return self->setFixedCellAlignemnt(col, row, (char)align);
+    }
+  }
+#endif
+
   long      setCellLock(WSCbool lock);
   long      setCellLock(long col, long row, WSCbool lock);
   WSCbool   getCellLock(long col, long row);
@@ -1575,8 +1736,8 @@ public:
 
 //-----------------------------------------------------------------------------
 #ifndef MSW //UNIX - Xwindow System
+#ifndef BTRON
 //-----------------------------------------------------------------------------
-
 #define WSK_BackSpace   0xFF08
 #define WSK_Tab         0xFF09
 #define WSK_Linefeed    0xFF0A
@@ -1848,6 +2009,280 @@ public:
 #define WSK_braceright          0x07d
 #define WSK_asciitilde          0x07e
 
+//-----------------------------------------------------------------------------
+#else //BTRON System
+//-----------------------------------------------------------------------------
+#define WSK_BackSpace		0x0008
+#define WSK_Tab		0x0009
+#define WSK_Linefeed		0x000a
+#define WSK_Clear		-1
+#define WSK_Return		0x000a
+#define WSK_Pause		-1
+#define WSK_Scroll_Lock		-1
+#define WSK_Sys_Req		-1
+#define WSK_Escape		0x0018
+#define WSK_Delete		0x007f
+
+#define WSK_Kanji		-1
+#define WSK_Muhenkan		-1
+#define WSK_Henkan_Mode		-1
+#define WSK_Henkan		0x001e
+#define WSK_Romaji		-1
+#define WSK_Hiragana		-1
+#define WSK_Katakana		-1
+#define WSK_Hiragana_Katakana		-1
+#define WSK_Zenkaku		-1
+#define WSK_Hankaku		-1
+#define WSK_Zenkaku_Hankaku		-1
+#define WSK_Touroku		-1
+#define WSK_Massyo		-1
+#define WSK_Kana_Lock		-1
+#define WSK_Kana_Shift		-1
+#define WSK_Eisu_Shift		-1
+#define WSK_Eisu_toggle		-1
+
+#define WSK_Home		0x1245
+#define WSK_Left		0x0103
+#define WSK_Up		0x0100
+#define WSK_Right		0x0102
+#define WSK_Down		0x0101
+#define WSK_Prior		-1
+#define WSK_Page_Up		0x1246
+#define WSK_Page_Down		0x1247
+#define WSK_End		0x125e
+//#define WSK_Begin		WSK_Begin
+
+//#define WSK_Select		WSK_Select
+#define WSK_Print		0x1148
+#define WSK_Execute		-1
+#define WSK_Insert		-1
+//#define WSK_Undo		WSK_Undo
+//#define WSK_Redo		WSK_Redo
+//#define WSK_Menu		WSK_Menu
+//#define WSK_Find		WSK_Find
+#define WSK_Cancel		0x0018
+#define WSK_Help		-1
+#define WSK_Break		-1
+//#define WSK_Mode_switch		WSK_Mode_switch
+//#define WSK_script_switch		WSK_script_switch
+#define WSK_Num_Lock		-1
+
+#define WSK_KP_Space		-1
+#define WSK_KP_Tab		-1
+#define WSK_KP_Enter		0x000a
+#define WSK_KP_F1		-1
+#define WSK_KP_F2		-1
+#define WSK_KP_F3		-1
+#define WSK_KP_F4		-1
+#define WSK_KP_Home		0x1245
+#define WSK_KP_Left		0x0103
+#define WSK_KP_Up		0x0100
+#define WSK_KP_Right		0x0102
+#define WSK_KP_Down		0x0101
+#define WSK_KP_Prior		-1
+#define WSK_KP_Page_Up		0x1246
+#define WSK_KP_Next		-1
+#define WSK_KP_Page_Down		0x1247
+#define WSK_KP_End		0x125e
+#define WSK_KP_Begin		-1
+#define WSK_KP_Insert		-1
+#define WSK_KP_Delete		-1
+#define WSK_KP_Equal		0x2161
+#define WSK_KP_Multiply		0x2176
+#define WSK_KP_Add		0x215c
+#define WSK_KP_Separator		0x2124
+#define WSK_KP_Subtract		0x215d
+#define WSK_KP_Decimal		0x2125
+#define WSK_KP_Divide		0x213f
+
+#define WSK_KP_0		(0x2330 +0)
+#define WSK_KP_1		(0x2330 +1)
+#define WSK_KP_2		(0x2330 +2)
+#define WSK_KP_3		(0x2330 +3)
+#define WSK_KP_4		(0x2330 +4)
+#define WSK_KP_5		(0x2330 +5)
+#define WSK_KP_6		(0x2330 +6)
+#define WSK_KP_7		(0x2330 +7)
+#define WSK_KP_8		(0x2330 +8)
+#define WSK_KP_9		(0x2330 +9)
+
+#define WSK_F1		0x1161
+#define WSK_F2		0x1162
+#define WSK_F3		0x1163
+#define WSK_F4		0x1164
+#define WSK_F5		0x1165
+#define WSK_F6		0x1166
+#define WSK_F7		0x1167
+#define WSK_F8		0x1168
+#define WSK_F9		0x1169
+#define WSK_F10		0x116a
+#define WSK_F11		0x116b
+#define WSK_L1		0x116b
+#define WSK_F12		0x116c
+#define WSK_L2		0x116c
+#define WSK_F13		-1
+#define WSK_L3		-1
+#define WSK_F14		-1
+#define WSK_L4		-1
+#define WSK_F15		-1
+#define WSK_L5		-1
+#define WSK_F16		-1
+#define WSK_L6		-1
+#define WSK_F17		-1
+#define WSK_L7		-1
+#define WSK_F18		-1
+#define WSK_L8		-1
+#define WSK_F19		-1
+#define WSK_L9		-1
+#define WSK_F20		-1
+#define WSK_L10		-1
+#define WSK_F21		-1
+#define WSK_R1		-1
+#define WSK_F22		-1
+#define WSK_R2		-1
+#define WSK_F23		-1
+#define WSK_R3		-1
+#define WSK_F24		-1
+#define WSK_R4		-1
+#define WSK_F25		-1
+#define WSK_R5		-1
+#define WSK_F26		-1
+#define WSK_R6		-1
+#define WSK_F27		-1
+#define WSK_R7		-1
+#define WSK_F28		-1
+#define WSK_R8		-1
+#define WSK_F29		-1
+#define WSK_R9		-1
+#define WSK_F30		-1
+#define WSK_R10		-1
+#define WSK_F31		-1
+#define WSK_R11		-1
+#define WSK_F32		-1
+#define WSK_R12		-1
+#define WSK_F33		-1
+#define WSK_R13		-1
+#define WSK_F34		-1
+#define WSK_R14		-1
+#define WSK_F35		-1
+#define WSK_R15		-1
+
+#define WSK_Shift_L		0x1003
+#define WSK_Shift_R		0x1002
+#define WSK_Control_L		-1
+#define WSK_Control_R		-1
+#define WSK_Caps_Lock		-1
+#define WSK_Shift_Lock		-1
+
+#define WSK_Meta_L		-1
+#define WSK_Meta_R		-1
+#define WSK_Alt_L		-1
+#define WSK_Alt_R		-1
+#define WSK_Super_L		-1
+#define WSK_Super_R		-1
+#define WSK_Hyper_L		-1
+#define WSK_Hyper_R		-1
+
+#define WSK_space		0x2121
+#define WSK_exclam		0x212a
+#define WSK_quotedbl		0x2149
+#define WSK_numbersign		0x2174
+#define WSK_dollar		0x2170
+#define WSK_percent		0x2173
+#define WSK_ampersand		0x2175
+#define WSK_apostrophe		0x2147
+#define WSK_quoteright		0x212e
+#define WSK_parenleft		0x214a
+#define WSK_parenright		0x214b
+#define WSK_asterisk		0x2176
+#define WSK_plus		0x215c
+#define WSK_comma		0x2124
+#define WSK_minus		0x215d
+#define WSK_period		0x2125
+#define WSK_slash		0x213f
+#define WSK_0		(0x2330 +0)
+#define WSK_1		(0x2330 +1)
+#define WSK_2		(0x2330 +2)
+#define WSK_3		(0x2330 +3)
+#define WSK_4		(0x2330 +4)
+#define WSK_5		(0x2330 +5)
+#define WSK_6		(0x2330 +6)
+#define WSK_7		(0x2330 +7)
+#define WSK_8		(0x2330 +8)
+#define WSK_9		(0x2330 +9)
+#define WSK_colon		0x2127
+#define WSK_semicolon		0x2128
+#define WSK_less		0x2163
+#define WSK_equal		0x2161
+#define WSK_greater		0x2164
+#define WSK_question		0x2129
+#define WSK_at		0x2177
+#define WSK_A		(0x2341 +0)
+#define WSK_B		(0x2341 +1)
+#define WSK_C		(0x2341 +2)
+#define WSK_D		(0x2341 +3)
+#define WSK_E		(0x2341 +4)
+#define WSK_F		(0x2341 +5)
+#define WSK_G		(0x2341 +6)
+#define WSK_H		(0x2341 +7)
+#define WSK_I		(0x2341 +8)
+#define WSK_J		(0x2341 +9)
+#define WSK_K		(0x2341 +10)
+#define WSK_L		(0x2341 +11)
+#define WSK_M		(0x2341 +12)
+#define WSK_N		(0x2341 +13)
+#define WSK_O		(0x2341 +14)
+#define WSK_P		(0x2341 +15)
+#define WSK_Q		(0x2341 +16)
+#define WSK_R		(0x2341 +17)
+#define WSK_S		(0x2341 +18)
+#define WSK_T		(0x2341 +19)
+#define WSK_U		(0x2341 +20)
+#define WSK_V		(0x2341 +21)
+#define WSK_W		(0x2341 +22)
+#define WSK_X		(0x2341 +23)
+#define WSK_Y		(0x2341 +24)
+#define WSK_Z		(0x2341 +25)
+#define WSK_bracketleft		0x214e
+#define WSK_backslash		0x216f
+#define WSK_bracketright		0x214f
+#define WSK_asciicircum		0x2130
+#define WSK_underscore		0x2132
+//#define WSK_grave		TK_RQUT
+//#define WSK_quoteleft		TK_RQUT
+#define WSK_a		(0x2361 +0)
+#define WSK_b		(0x2361 +1)
+#define WSK_c		(0x2361 +2)
+#define WSK_d		(0x2361 +3)
+#define WSK_e		(0x2361 +4)
+#define WSK_f		(0x2361 +5)
+#define WSK_g		(0x2361 +6)
+#define WSK_h		(0x2361 +7)
+#define WSK_i		(0x2361 +8)
+#define WSK_j		(0x2361 +9)
+#define WSK_k		(0x2361 +10)
+#define WSK_l		(0x2361 +11)
+#define WSK_m		(0x2361 +12)
+#define WSK_n		(0x2361 +13)
+#define WSK_o		(0x2361 +14)
+#define WSK_p		(0x2361 +15)
+#define WSK_q		(0x2361 +16)
+#define WSK_r		(0x2361 +17)
+#define WSK_s		(0x2361 +18)
+#define WSK_t		(0x2361 +19)
+#define WSK_u		(0x2361 +20)
+#define WSK_v		(0x2361 +21)
+#define WSK_w		(0x2361 +22)
+#define WSK_x		(0x2361 +23)
+#define WSK_y		(0x2361 +24)
+#define WSK_z		(0x2361 +25)
+#define WSK_braceleft		0x2150
+#define WSK_bar		0x2143
+#define WSK_braceright		0x2151
+#define WSK_asciitilde		0x2131
+
+#endif //#ifndef BTRON
+
 //-----------------------------------------------------------------------------
 #else //MS Windows System
 //-----------------------------------------------------------------------------
@@ -2115,7 +2550,7 @@ public:
 #define WSK_braceright			0x17d
 #define WSK_asciitilde			0x17e
 
-#endif
+#endif //#ifndef MSW
 
 #define WS_LIST_TREE_OPEN "list-tr-open"
 #define WS_LIST_TREE_OPEN_CH "list-tr-open-ch"
@@ -2124,4 +2559,4 @@ public:
 #define WS_LIST_LABEL_LIST "list-lb-list"
 #define WS_LIST_TOP "list-lb-top"
 
-#endif
+#endif //#ifndef mpfc_H
