@@ -122,8 +122,8 @@ WSCsheet::WSCsheet(WSCbase* base, char* objname) : WSCform(base, objname)
     WSMFpropertyCreate(WSNfixedCols, long, _fixedCols, WSSfixedCols);
     WSMFpropertyCreate(WSNgridLines, short, _gridLines, WSSgridLines);
     WSMFpropertySetSelection(WSRgridLines, WSRgridLinesD);
-    WSMFpropertyCreate(WSNcelldefWidth, long, _colWidth, WSScellDefWidth);
-    WSMFpropertyCreate(WSNcelldefHeight, long, _colHeight, WSScellDefHeight);
+    WSMFpropertyCreate(WSNcellDefWidth, long, _colWidth, WSScellDefWidth);
+    WSMFpropertyCreate(WSNcellDefHeight, long, _colHeight, WSScellDefHeight);
     WSMFpropertyCreate(WSNfixedColHeight, long, _fixedColHeight, WSSfixedColHeight);
     WSMFpropertyCreate(WSNcellDefFont, short, _font, WSScellDefFont);
     WSMFpropertyCreate(WSNfixedCellFont, short, _fixedFont, WSSfixedCellFont);
@@ -177,8 +177,8 @@ WSMFproperty(WSCsheet, WSNcols, long, _cols, WSCSHEET_DEF_COLS);
 WSMFproperty(WSCsheet, WSNfixedRows, long, _fixedRows, WSCSHEET_DEF_FIXEDROWS);
 WSMFproperty(WSCsheet, WSNfixedCols, long, _fixedCols, WSCSHEET_DEF_FIXEDCOLS);
 WSMFproperty(WSCsheet, WSNgridLines, short, _gridLines, WSCSHEET_BOTH);
-WSMFproperty(WSCsheet, WSNcelldefWidth, long, _colWidth, WSCSHEET_DEF_COL_WIDTH);
-WSMFproperty(WSCsheet, WSNcelldefHeight, long, _colHeight, WSCSHEET_DEF_ROW_HEIGHT);
+WSMFproperty(WSCsheet, WSNcellDefWidth, long, _colWidth, WSCSHEET_DEF_COL_WIDTH);
+WSMFproperty(WSCsheet, WSNcellDefHeight, long, _colHeight, WSCSHEET_DEF_ROW_HEIGHT);
 WSMFproperty(WSCsheet, WSNfixedColHeight, long, _fixedColHeight, WSCSHEET_DEF_ROW_HEIGHT);
 WSMFproperty(WSCsheet, WSNcellDefFont, short, _font, (char)WSGIappFontSet()->getDefaultFontNo());
 WSMFproperty(WSCsheet, WSNfixedCellFont, short, _fixedFont, (char)WSGIappFontSet()->getDefaultFontNo());
@@ -208,59 +208,59 @@ WSMFproperty(WSCsheet, WSNdbCellTitle, WSCbool, _dbCellTitle, True);
 
 void WSCsheet::setWorkWSNrows(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setRowCount(val);
 	}
 }
 void WSCsheet::getWorkWSNrows(long*){}
 void WSCsheet::setWorkWSNcols(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setColumnCount(val);
 	}
 }
 void WSCsheet::getWorkWSNcols(long*){}
 void WSCsheet::setWorkWSNfixedRows(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setFixedRowCount(val);
 	}
 }
 void WSCsheet::getWorkWSNfixedRows(long*){}
 void WSCsheet::setWorkWSNfixedCols(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setFixedColumnCount(val);
 	}
 }
 void WSCsheet::getWorkWSNfixedCols(long*){}
 void WSCsheet::setWorkWSNgridLines(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setGridLines(val);
 	}
 }
 void WSCsheet::getWorkWSNgridLines(short*){}
-void WSCsheet::setWorkWSNcelldefWidth(long val)
+void WSCsheet::setWorkWSNcellDefWidth(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		_colWidth = val;
 		needUpdate();
 	}
 }
-void WSCsheet::getWorkWSNcelldefWidth(long*){}
-void WSCsheet::setWorkWSNcelldefHeight(long val)
+void WSCsheet::getWorkWSNcellDefWidth(long*){}
+void WSCsheet::setWorkWSNcellDefHeight(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		_colHeight = val;
 		needUpdate();
 	}
 }
-void WSCsheet::getWorkWSNcelldefHeight(long*){}
+void WSCsheet::getWorkWSNcellDefHeight(long*){}
 
 void WSCsheet::setWorkWSNfixedColHeight(long val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		_fixedColHeight = val;
 		needUpdate();
 	}
@@ -268,14 +268,14 @@ void WSCsheet::setWorkWSNfixedColHeight(long val)
 void WSCsheet::getWorkWSNfixedColHeight(long*){}
 void WSCsheet::setWorkWSNcellDefFont(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setCellDefFont(val);
 	}
 }
 void WSCsheet::getWorkWSNcellDefFont(short*){}
 void WSCsheet::setWorkWSNfixedCellFont(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setFixedCellFont(val);
 	}
 }
@@ -284,7 +284,7 @@ void WSCsheet::setWorkWSNeditEnabled(WSCbool val){}
 void WSCsheet::getWorkWSNeditEnabled(WSCbool*){}
 void WSCsheet::setWorkWSNhautoNumber(WSCbool val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setHorzAutoNumber(val);
 		needUpdate();
 	}
@@ -292,7 +292,7 @@ void WSCsheet::setWorkWSNhautoNumber(WSCbool val)
 void WSCsheet::getWorkWSNhautoNumber(WSCbool*){}
 void WSCsheet::setWorkWSNvautoNumber(WSCbool val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setVertAutoNumber(val);
 		needUpdate();
 	}
@@ -300,42 +300,42 @@ void WSCsheet::setWorkWSNvautoNumber(WSCbool val)
 void WSCsheet::getWorkWSNvautoNumber(WSCbool*){}
 void WSCsheet::setWorkWSNsheetBackColor(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setSheetBkColor(val);
 	}
 }
 void WSCsheet::getWorkWSNsheetBackColor(short*){}
 void WSCsheet::setWorkWSNcellDefBackColor(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setCellDefBkColor(val);
 	}
 }
 void WSCsheet::getWorkWSNcellDefBackColor(short*){}
 void WSCsheet::setWorkWSNcellDefTextColor(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setCellDefTextColor(val);
 	}
 }
 void WSCsheet::getWorkWSNcellDefTextColor(short*){}
 void WSCsheet::setWorkWSNfixedBackColor(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setFixedBkColor(val);
 	}
 }
 void WSCsheet::getWorkWSNfixedBackColor(short*){}
 void WSCsheet::setWorkWSNfixedTextColor(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setFixedTextColor(val);
 	}
 }
 void WSCsheet::getWorkWSNfixedTextColor(short*){}
 void WSCsheet::setWorkWSNgridColor(short val)
 {
-	if(_initialized != False && _dev != NULL){
+	if(_initialized != False){
 		setGridColor(val);
 	}
 }
@@ -1002,6 +1002,18 @@ long WSCsheet::setFixedRowCount(long fixedRows)
 		return WS_ERR;
 	}
 	_fixedRows = fixedRows;
+	if(_focusRow < _fixedRows) {
+		_focusRow = _fixedRows;
+	}
+	if(_startRow < _fixedRows) {
+		_startRow = _fixedRows;
+	}
+	if(_endRow < _fixedRows) {
+		_endRow = _fixedRows;
+	}
+	if(_topVisibleRow < _fixedRows) {
+		_topVisibleRow = _fixedRows;
+	}
 	if(_fixedRows <= _rows) {
 		return WS_NO_ERR;
 	}
@@ -1013,6 +1025,18 @@ long WSCsheet::setFixedColumnCount(long fixedCols)
 		return WS_ERR;
 	}
 	_fixedCols = fixedCols;
+	if(_focusCol < _fixedCols) {
+		_focusCol = _fixedCols;
+	}
+	if(_startCol < _fixedCols) {
+		_startCol = _fixedCols;
+	}
+	if(_endCol < _fixedCols) {
+		_endCol = _fixedCols;
+	}
+	if(_topVisibleCol < _fixedCols) {
+		_topVisibleCol = _fixedCols;
+	}
 	if(_fixedCols <= _cols) {
 		return WS_NO_ERR;
 	}
@@ -2865,8 +2889,7 @@ long WSCsheet::setDbStmtString(WSCstring& stmt)
 }
 long WSCsheet::dbRefresh()
 {
-	dbDataSet();
-	return WS_NO_ERR;
+	return dbDataSet();;
 }
 ///////////////////////////////////////////////////////////////////////////////
 // database local function
@@ -2931,11 +2954,16 @@ long WSCsheet::setFixedCelldbItem(WSCdbRecord* rs)
 }
 long WSCsheet::dbDataSet()
 {
+	_dbError = "";
+	
 	dbConnect();
 
 	if(!_vdb) {
 		return WS_NO_ERR;
 	}
+	if(*((char*)_vdb->getProperty(WSNhostname)) == '\0') {
+		return WS_NO_ERR;
+	}
 
 	WSDfont* font = WSGIappFontSet()->getFont(_fixedFont);
 
@@ -2993,48 +3021,69 @@ long WSCsheet::dbDataSet()
 	setHorzAutoNumber(False);
 
 //	clear(2, 2);
-
+	long ret = WS_NO_ERR;
+	
 	WSCdbRecord rs(_vdb);
 
 	if(_vdb->open() == WS_NO_ERR) {
 
-		rs.open(stmt);
+		if(rs.open(stmt) == WS_NO_ERR) {
+			setColumnCount(rs.getColCount() + getFixedColumnCount()+1);
+			long rows = rs.getRowCount();
+			if(rows < 0) {
+				rows = 0;
+			}
+			dataClear(rs.getColCount() + getFixedColumnCount(), rows + getFixedRowCount());
 
-		setColumnCount(rs.getColCount() + getFixedColumnCount()+1);
-		dataClear(rs.getColCount() + getFixedColumnCount(), rs.getRowCount() + getFixedRowCount());
+			if(_dbCellTitle) {
+				setFixedCelldbItem(&rs);
+			}
+			
+			WSCstring str;
+			char* val = new char[32767*2];
+			long col = 0;
+			long row = 1;
+			long width;
+			long fixedCols = getFixedColumnCount();
+			
+			while (!rs.isEOF()) {
+				for(col = 0; col < rs.getColCount(); col++) {
+					rs.getColStringValue(col, val);
+					if(row >= getRowCount()) {
+						setRowCount(row+1);
+					}
+					setCellText(col + fixedCols, row, val);
 
-		if(_dbCellTitle) {
-			setFixedCelldbItem(&rs);
-		}
-		
-		WSCstring str;
-		char* val = new char[32767*2];
-		long col = 0;
-		long row = 1;
-		long width;
-		long fixedCols = getFixedColumnCount();
-		
-		while (!rs.isEOF()) {
-			for(col = 0; col < rs.getColCount(); col++) {
-				rs.getColStringValue(col, val);
-				setCellText(col + fixedCols, row, val);
-
-				if(_dbCellAutoResize) {
-					str = val;
-					width = getColumnWidth(col + fixedCols);
-					if(width < (font->getStringWidth(&str)+4)) {
-						setColumnWidth(col + fixedCols, font->getStringWidth(&str)+4);
+					if(_dbCellAutoResize) {
+						str = val;
+						width = getColumnWidth(col + fixedCols);
+						if(width < (font->getStringWidth(&str)+4)) {
+							setColumnWidth(col + fixedCols, font->getStringWidth(&str)+4);
+						}
 					}
 				}
+				row++;
+				rs.moveNext();
 			}
-			row++;
-			rs.moveNext();
+			delete[] val;
+		} else {
+			_dbError = rs.getErrorMessage();
+			if(_dbCellTitle) {
+				dataClear();
+			} else {
+				dataClear(getColumnCount(), 2);
+			}
+			ret = WS_ERR;
 		}
 		rs.close();
-
-		delete[] val;
 	} else {
-		dataClear();
+		_dbError = _vdb->getErrorMessage();
+		if(_dbCellTitle) {
+			dataClear();
+		} else {
+			dataClear(getColumnCount(), 2);
+		}
+		ret = WS_ERR;
 	}
 	adjustScrollBar();
 
@@ -3042,7 +3091,7 @@ long WSCsheet::dbDataSet()
 
 	draw();
 
-	return WS_NO_ERR;
+	return ret;
 }
 long WSCsheet::dataClear(long col, long row)
 {
@@ -3078,3 +3127,7 @@ long WSCsheet::dataClear(long col, long row)
 
 	return WS_NO_ERR;
 }
+WSCstring WSCsheet::getDbErrorMsg()
+{
+	return _dbError;
+}
