@@ -435,6 +435,8 @@ dbprintf("WndProc %s:%d done4\n",__FILE__,__LINE__);
     WSCbool ret = WSGIwinKeyboard()->_dispatch_key(&msg);
     if (ret == False){
 dbprintf("WndProc %s:%d done5\n",__FILE__,__LINE__);
+      WSGIappObjectList()->execInitialize();
+      WSGIappObjectList()->execUpdate();
       return 0;
     }
   }
