@@ -316,7 +316,17 @@ WSMFtrace("XXZZim1 here3\n");
             ptr2 +=3;
           }
         }
-      }else{
+    }else if (jinf.jpeg_color_space == JCS_GRAYSCALE){
+      for(i=0; i<height; i++){
+        for(j=0; j<width; j++){
+          long base = (height - 1 -i)*bytes_per_line + j*3;
+          ((unsigned char*)data)[base +2] = buf[ptr];
+          ((unsigned char*)data)[base +1] = buf[ptr];
+          ((unsigned char*)data)[base] = buf[ptr++];
+          ptr2 +=3;
+        }
+      }
+    }else{
 WSMFtrace("XXZZim1 here1\n");
         for(i=0; i<height; i++){
           for(j=0; j<width; j++){
@@ -358,6 +368,8 @@ WSMFtrace("XXZZim1 setDIBits error %s\n",filename);
 void png_cexcept_error(png_structp png_ptr, png_const_charp msg){
    if(png_ptr){
      fprintf(stderr, "png file read error: %s\n", msg);
+     bool*  png_err_flag_ptr = (bool*) png_get_error_ptr(png_ptr);
+     *png_err_flag_ptr = true;
    }
 }
 
@@ -387,24 +399,36 @@ HBITMAP PngReadFileToBitmap(char* filename,WSCulong* w,WSCulong* h,HBITMAP* mask
     fread(sig, 1, 8, ifile);
     if (!png_check_sig(sig, 8)){
 //printf("XXZZ Png err2\n");
+      fclose(ifile);
       return NULL;
     }
-    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL,
+    bool png_err_flag = false;
+    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
+      (png_voidp) &png_err_flag,
       (png_error_ptr)png_cexcept_error, (png_error_ptr)NULL);
     if (png_ptr == NULL){
 //printf("XXZZ Png err3\n");
+      fclose(ifile);
       return NULL;
     } 
     png_infop info_ptr = png_create_info_struct(png_ptr);
     if (info_ptr == NULL){
 //printf("XXZZ Png err4\n");
       png_destroy_read_struct(&png_ptr, NULL, NULL);
+      fclose(ifile);
       return NULL;
     }
 
     png_init_io(png_ptr, ifile);
     png_set_sig_bytes(png_ptr, 8);
     png_read_info(png_ptr, info_ptr);
+
+    if (png_err_flag != false)  {
+      png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
+      fclose(ifile);
+      return NULL;
+    }
+
     png_uint_32 png_width;
     png_uint_32 png_height;
     int png_depth;
@@ -415,6 +439,55 @@ HBITMAP PngReadFileToBitmap(char* filename,WSCulong* w,WSCulong* h,HBITMAP* mask
     if (png_depth == 16){
       png_set_strip_16(png_ptr);
     }
+#if 1 // replace new logic
+    if (png_depth < 8)
+        png_set_packing(png_ptr);
+    //  Transformations
+    //  from readpng2_info_callback() in contrib/gregbook/readpng2.c
+    if (png_color_type == PNG_COLOR_TYPE_PALETTE)
+        png_set_expand(png_ptr);
+    if (png_color_type == PNG_COLOR_TYPE_GRAY && png_depth < 8)
+        png_set_expand(png_ptr);
+    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
+        png_set_expand(png_ptr);
+    if (png_color_type == PNG_COLOR_TYPE_GRAY ||
+        png_color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
+        png_set_gray_to_rgb(png_ptr);
+
+    // Ignore background color with transparent and alpha
+    // png_get_bKGD(), png_set_background()
+
+    // Gamma conversion
+    //  example.c 
+    int intent;
+    //screen_gamma = 2.2; // A good guess for a PC monitor
+    //                    //   in a bright office or a dim room
+    //screen_gamma = 2.0; // A good guess for a PC monitor
+    //                    //   in a dark room
+    const double screen_gamma = 2.2;
+    if (png_get_sRGB(png_ptr, info_ptr, &intent))
+       png_set_gamma(png_ptr, screen_gamma, 0.45455);
+    else
+    {
+       double image_gamma;
+       if (png_get_gAMA(png_ptr, info_ptr, &image_gamma))
+          png_set_gamma(png_ptr, screen_gamma, image_gamma);
+       else
+          png_set_gamma(png_ptr, screen_gamma, 0.45455);
+    }
+
+    // after the transformations have been registered update info_ptr data
+    png_read_update_info(png_ptr, info_ptr);
+
+    // need this code?
+    // get again width, height and the new bit-depth and color-type
+    png_get_IHDR(png_ptr, info_ptr, &png_width, &png_height, &png_depth,
+            &png_color_type, NULL, NULL, NULL);
+
+    png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
+    png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
+
+#else // replace 
     png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
     png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
 //printf("load png row_bytes=%d channels=%d depth=%d\n",png_row_bytes,png_channels,png_depth);
@@ -466,6 +539,7 @@ HBITMAP PngReadFileToBitmap(char* filename,WSCulong* w,WSCulong* h,HBITMAP* mask
       png_row_bytes = png_width * 3;
       png_channels = 3;
     }
+#endif
 
     long rwidth = png_width;
     long rheight = png_height;
@@ -491,6 +565,7 @@ fprintf(stderr,"png read error: out of memory..\n");
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
 //printf("XXZZ Png err6\n");
       fclose(ifile);
+      delete[] buf;
       return NULL;
     }
     int i;
@@ -541,6 +616,7 @@ fprintf(stderr,"png read error: out of memory..\n");
     long j;
 //    if (components == 3 || components == 4){
       for(i=0; i<height; i++){
+        ptr = i * png_row_bytes;
         for(j=0; j<width; j++){
           long base = (height - 1 -i)*bytes_per_line + j*3;
           ((unsigned char*)data)[base +2] = buf[ptr++];
@@ -553,7 +629,6 @@ fprintf(stderr,"png read error: out of memory..\n");
             ((unsigned char*)data2)[base] = a;
           }
         }
-        ptr = i * png_row_bytes;
       }
 //    }else{
 //      for(i=0; i<height; i++){
