@@ -884,8 +884,114 @@ long WSDwindraw::drawFillPoly(WSCpoint* points,long num){
   return WS_NO_ERR;
 }
 
+void _draw_stretch_image_with_mask(HDC hdc,HDC mdc,HDC mdc2,HBITMAP pix,HBITMAP pix2,HPALETTE pl,
+                                   int x,int y,int w,int h, int sx,int sy,int iw,int ih){
+      HDC mdc3 = CreateCompatibleDC(hdc);
+      HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
+      RealizePalette(mdc3);
+      HBITMAP pix3 = CreateCompatibleBitmap(hdc,iw,ih);
+      HBITMAP pxbk3 = (HBITMAP)SelectObject(mdc3,(void*)pix3);
+      SetMapMode(mdc3,GetMapMode(hdc));
+      StretchBlt(mdc3,sx,sy,iw,ih,hdc,x,y,w,h,SRCCOPY);
+
+      BITMAPINFO bmi;
+      bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+      bmi.bmiHeader.biWidth=iw;
+      bmi.bmiHeader.biHeight=ih;
+      bmi.bmiHeader.biPlanes=1;
+      bmi.bmiHeader.biBitCount=24;
+      bmi.bmiHeader.biCompression=BI_RGB;
+      bmi.bmiHeader.biSizeImage=0;
+      bmi.bmiHeader.biXPelsPerMeter=0;
+      bmi.bmiHeader.biYPelsPerMeter=0;
+      bmi.bmiHeader.biClrUsed=0;
+      bmi.bmiHeader.biClrImportant=0;
+      int bytes_per_line = (iw+1)*3 & ~3;
+
+      unsigned char* buf1 = new unsigned char[bytes_per_line * ih];
+      unsigned char* buf2 = new unsigned char[bytes_per_line * ih];
+      unsigned char* buf3 = new unsigned char[bytes_per_line * ih];
+
+      GetDIBits(mdc,pix,0,ih,(void*)buf3,&bmi,DIB_RGB_COLORS);
+      GetDIBits(mdc2,pix2,0,ih,(void*)buf2,&bmi,DIB_RGB_COLORS);
+      GetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+
+      int i,j;
+      for(i=0; i<ih; i++){
+        long ptr = i*bytes_per_line;
+        for(j=0; j<bytes_per_line; j++){
+          long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+          buf1[ptr] = val;
+          ptr++;
+        }
+      }
+      SetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+
+      StretchBlt(hdc,x,y,w,h,mdc3,0,0,iw,ih,SRCCOPY);
+
+      SelectObject(mdc3,pxbk3);
+      DeleteObject(pix3);
+      DeleteDC(mdc3);
+      delete buf1;
+      delete buf2;
+      delete buf3;
+}
+
+void _draw_image_with_mask(HDC hdc,HDC mdc,HDC mdc2,HBITMAP pix,HBITMAP pix2,HPALETTE pl,
+                                   int x,int y,int w,int h){
+        HDC mdc3 = CreateCompatibleDC(hdc);
+        HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
+        RealizePalette(mdc3);
+        HBITMAP pix3 = CreateCompatibleBitmap(hdc,w,h);
+        HBITMAP pxbk3 = (HBITMAP)SelectObject(mdc3,(void*)pix3);
+        SetMapMode(mdc3,GetMapMode(hdc));
+        BitBlt(mdc3,0,0,w,h,hdc,x,y,SRCCOPY);
+
+        BITMAPINFO bmi;
+        bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+        bmi.bmiHeader.biWidth=w;
+        bmi.bmiHeader.biHeight=h;
+        bmi.bmiHeader.biPlanes=1;
+        bmi.bmiHeader.biBitCount=24;
+        bmi.bmiHeader.biCompression=BI_RGB;
+        bmi.bmiHeader.biSizeImage=0;
+        bmi.bmiHeader.biXPelsPerMeter=0;
+        bmi.bmiHeader.biYPelsPerMeter=0;
+        bmi.bmiHeader.biClrUsed=0;
+        bmi.bmiHeader.biClrImportant=0;
+
+        int bytes_per_line = (w+1)*3 & ~3;
+        unsigned char* buf1 = new unsigned char[bytes_per_line * h];
+        unsigned char* buf2 = new unsigned char[bytes_per_line * h];
+        unsigned char* buf3 = new unsigned char[bytes_per_line * h];
+        GetDIBits(mdc3,pix3,0,h,(void*)buf1,&bmi,DIB_RGB_COLORS);
+        GetDIBits(mdc2,pix2,0,h,(void*)buf2,&bmi,DIB_RGB_COLORS);
+        GetDIBits(mdc,pix,0,h,(void*)buf3,&bmi,DIB_RGB_COLORS);
+
+        int i,j;
+        for(i=0; i<h; i++){
+          long ptr = i*bytes_per_line;
+          for(j=0; j<bytes_per_line; j++){
+            long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+            buf1[ptr] = val;
+            ptr++;
+          }
+        }
+
+        SetDIBits(mdc3,pix3,0,h,(void*)buf1,&bmi,DIB_RGB_COLORS);
+        BitBlt(hdc,x,y,w,h,mdc3,0,0,SRCCOPY);
+
+
+        SelectObject(mdc3,pxbk3);
+        DeleteObject(pix3);
+        DeleteDC(mdc3);
+        delete buf1;
+        delete buf2;
+        delete buf3;
+}
+
 long WSDwindraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDimage* image){
-//long XXZZXXZZ;
+
   if (_printer_hdc != 0){
     x = (short)((x - _pr_x) * _pr_scale_x);
     y = (short)((y - _pr_y) * _pr_scale_y);
@@ -904,7 +1010,9 @@ long WSDwindraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDi
   if (icon == 0){
     HDC mdc = CreateCompatibleDC(_hdc);
     HDC mdc2 = 0;
-    
+    if (pix2 != 0){
+      mdc2 = CreateCompatibleDC(_hdc);
+    }
     HPALETTE pl = WSGIwinAppDev()->getPalette();
     HPALETTE plbk = 0;
     HPALETTE plbk2 = 0;
@@ -928,11 +1036,68 @@ long WSDwindraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDi
     WSCulong iw = image->getImageWidth();
     WSCulong ih = image->getImageHeight();
     SetStretchBltMode(_hdc,STRETCH_DELETESCANS);
+
     if (pix2 == 0){
       StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCCOPY);
     }else{
-      StretchBlt(_hdc,x,y,w,h,mdc2,0,0,iw,ih,SRCAND);
-      StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCPAINT);
+//      StretchBlt(_hdc,x,y,w,h,mdc2,0,0,iw,ih,SRCAND);
+//      StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCPAINT);
+#if 0
+      HDC mdc3 = CreateCompatibleDC(_hdc);
+      HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
+      RealizePalette(mdc3);
+      HBITMAP pix3 = CreateCompatibleBitmap(_hdc,iw,ih);
+      HBITMAP pxbk3 = (HBITMAP)SelectObject(mdc3,(void*)pix3);
+      SetMapMode(mdc3,GetMapMode(_hdc));
+      StretchBlt(mdc3,0,0,iw,ih,_hdc,x,y,w,h,SRCCOPY);
+
+      BITMAPINFO bmi;
+      bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+      bmi.bmiHeader.biWidth=iw;
+      bmi.bmiHeader.biHeight=ih;
+      bmi.bmiHeader.biPlanes=1;
+      bmi.bmiHeader.biBitCount=24;
+      bmi.bmiHeader.biCompression=BI_RGB;
+      bmi.bmiHeader.biSizeImage=0;
+      bmi.bmiHeader.biXPelsPerMeter=0;
+      bmi.bmiHeader.biYPelsPerMeter=0;
+      bmi.bmiHeader.biClrUsed=0;
+      bmi.bmiHeader.biClrImportant=0;
+      int bytes_per_line = (iw+1)*3 & ~3;
+
+      unsigned char* buf1 = new unsigned char[bytes_per_line * ih];
+      unsigned char* buf2 = new unsigned char[bytes_per_line * ih];
+      unsigned char* buf3 = new unsigned char[bytes_per_line * ih];
+
+      GetDIBits(mdc,pix,0,ih,(void*)buf3,&bmi,DIB_RGB_COLORS);
+      GetDIBits(mdc2,pix2,0,ih,(void*)buf2,&bmi,DIB_RGB_COLORS);
+      GetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+
+      int i,j;
+      for(i=0; i<ih; i++){
+        long ptr = i*bytes_per_line;
+        for(j=0; j<bytes_per_line; j++){
+//printf("%d,%d a=0x%x  d=0x%x",j,i,buf2[ptr],buf1[ptr]);
+          long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+//          long val = buf3[ptr];
+          buf1[ptr] = val;
+//printf(" s=0x%x out=0x%x\n",buf3[ptr],buf1[ptr]);
+          ptr++;
+        }
+      }
+      SetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+
+      StretchBlt(_hdc,x,y,w,h,mdc3,0,0,iw,ih,SRCCOPY);
+
+      SelectObject(mdc3,pxbk3);
+      DeleteObject(pix3);
+      DeleteDC(mdc3);
+      delete buf1;
+      delete buf2;
+      delete buf3;
+#endif
+      _draw_stretch_image_with_mask(_hdc,mdc,mdc2,pix,pix2,pl,x,y,w,h,0,0,iw,ih);
+
     }
     if (pl != 0){
       SelectPalette(mdc,plbk,FALSE);
@@ -958,6 +1123,7 @@ long WSDwindraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* ima
   }
   HBITMAP pix = (HBITMAP)image->getValue1();
   HBITMAP pix2 = (HBITMAP)image->getValue3();
+  HBITMAP pix3 = (HBITMAP)0;
   if (pix == 0){
     return WS_NO_ERR;
   }
@@ -1051,31 +1217,78 @@ long WSDwindraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* ima
       yoff = (short)((yoff - _pr_y) * _pr_scale_y);
       dx = (short)((dx - _pr_x) * _pr_scale_x);
       dy = (short)((dy - _pr_y) * _pr_scale_y);
+      WSCulong iw2 = dw;
+      WSCulong ih2 = dh;
       dw = (WSCushort)(dw * _pr_scale_x);
       dh = (WSCushort)(dh * _pr_scale_y);
       SetStretchBltMode(_hdc,STRETCH_DELETESCANS);
       WSCulong iw = image->getImageWidth();
       WSCulong ih = image->getImageHeight();
       if (pix2 == 0){
-        StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw,ih,SRCCOPY);
+        StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw2,ih2,SRCCOPY);
       }else{
-        StretchBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x -xoff,dy-y-yoff,iw,ih,SRCAND);
-        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
-        BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
-        StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw,ih,SRCPAINT);
-        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+//        StretchBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x -xoff,dy-y-yoff,iw2,ih2,SRCAND);
+//        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+//        BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
+//        StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw2,ih2,SRCPAINT);
+//        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+
+        _draw_stretch_image_with_mask(_hdc,mdc,mdc2,pix,pix2,pl,dx,dy,dw,dh,
+                                      dx -x -xoff, dy -y -yoff,iw2,ih2);
       }
     }else{
       if (pix2 == 0){
         BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCCOPY);
       }else{
-        BitBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x -xoff,dy -y -yoff,SRCAND);
-        WSCulong iw = image->getImageWidth();
-        WSCulong ih = image->getImageHeight();
-        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
-        BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
-        BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCPAINT);
-        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+        HDC mdc3 = CreateCompatibleDC(_hdc);
+        HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
+        RealizePalette(mdc3);
+        HBITMAP pix3 = CreateCompatibleBitmap(_hdc,pw,ph);
+        HBITMAP pxbk3 = (HBITMAP)SelectObject(mdc3,(void*)pix3);
+        SetMapMode(mdc3,GetMapMode(_hdc));
+        BitBlt(mdc3,0,0,pw,ph,_hdc,x+xoff,y+yoff,SRCCOPY);
+
+        BITMAPINFO bmi;
+        bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+        bmi.bmiHeader.biWidth=pw;
+        bmi.bmiHeader.biHeight=ph;
+        bmi.bmiHeader.biPlanes=1;
+        bmi.bmiHeader.biBitCount=24;
+        bmi.bmiHeader.biCompression=BI_RGB;
+        bmi.bmiHeader.biSizeImage=0;
+        bmi.bmiHeader.biXPelsPerMeter=0;
+        bmi.bmiHeader.biYPelsPerMeter=0;
+        bmi.bmiHeader.biClrUsed=0;
+        bmi.bmiHeader.biClrImportant=0;
+
+        int bytes_per_line = (pw+1)*3 & ~3;
+        unsigned char* buf1 = new unsigned char[bytes_per_line * ph];
+        unsigned char* buf2 = new unsigned char[bytes_per_line * ph];
+        unsigned char* buf3 = new unsigned char[bytes_per_line * ph];
+        GetDIBits(mdc3,pix3,0,ph,(void*)buf1,&bmi,DIB_RGB_COLORS);
+        GetDIBits(mdc2,pix2,0,ph,(void*)buf2,&bmi,DIB_RGB_COLORS);
+        GetDIBits(mdc,pix,0,ph,(void*)buf3,&bmi,DIB_RGB_COLORS);
+
+        int i,j;
+        for(i=0; i<ph; i++){
+          long ptr = i*bytes_per_line;
+          for(j=0; j<bytes_per_line; j++){
+            long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+            buf1[ptr] = val;
+            ptr++;
+          }
+        }
+
+        SetDIBits(mdc3,pix3,0,ph,(void*)buf1,&bmi,DIB_RGB_COLORS);
+        BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x -xoff,dy -y -yoff,SRCCOPY);
+
+
+        SelectObject(mdc3,pxbk3);
+        DeleteObject(pix3);
+        DeleteDC(mdc3);
+        delete buf1;
+        delete buf2;
+        delete buf3;
       }
     }
     if (pl != 0){
@@ -1168,31 +1381,89 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
     sy = (short)((sy - _pr_y) * _pr_scale_y);
     dx = (short)((dx - _pr_x) * _pr_scale_x);
     dy = (short)((dy - _pr_y) * _pr_scale_y);
+    WSCulong iw2 = dw;
+    WSCulong ih2 = dh;
     dw = (WSCushort)(dw * _pr_scale_x);
     dh = (WSCushort)(dh * _pr_scale_y);
     SetStretchBltMode(_hdc,STRETCH_DELETESCANS);
     WSCulong iw = image->getImageWidth();
     WSCulong ih = image->getImageHeight();
     if (pix2 == 0){
-      StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw,ih,SRCCOPY);
+      StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw2,ih2,SRCCOPY);
     }else{
-      StretchBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x +sx,dy -y +sy,iw,ih,SRCAND);
-      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
-      BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
-      StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw,ih,SRCPAINT);
-      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+//      StretchBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x +sx,dy -y +sy,iw2,ih2,SRCAND);
+//      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+//      BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
+//      StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw2,ih2,SRCPAINT);
+//      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+
+      _draw_stretch_image_with_mask(_hdc,mdc,mdc2,pix,pix2,pl,dx,dy,dw,dh,
+                                      dx -x +sx, dy -y +sx,iw2,ih2);
     }
   }else{
     if (pix2 == 0){
       BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,SRCCOPY);
     }else{
-      BitBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x +sx,dy -y +sy,SRCAND);
+//      BitBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x +sx,dy -y +sy,SRCAND);
+//      WSCulong iw = image->getImageWidth();
+//      WSCulong ih = image->getImageHeight();
+//      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+//      BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
+//      BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,SRCPAINT);
+//      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+
       WSCulong iw = image->getImageWidth();
       WSCulong ih = image->getImageHeight();
-      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
-      BitBlt(mdc,0,0,iw,ih,mdc2,0,0,SRCAND);
-      BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,SRCPAINT);
-      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
+      HDC mdc3 = CreateCompatibleDC(_hdc);
+      HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
+      RealizePalette(mdc3);
+      HBITMAP pix3 = CreateCompatibleBitmap(_hdc,iw,ih);
+      HBITMAP pxbk3 = (HBITMAP)SelectObject(mdc3,(void*)pix3);
+      SetMapMode(mdc3,GetMapMode(_hdc));
+      BitBlt(mdc3,0,0,iw,ih,_hdc,x -sx,y - sy,SRCCOPY);
+
+      BITMAPINFO bmi;
+      bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+      bmi.bmiHeader.biWidth=iw;
+      bmi.bmiHeader.biHeight=ih;
+      bmi.bmiHeader.biPlanes=1;
+      bmi.bmiHeader.biBitCount=24;
+      bmi.bmiHeader.biCompression=BI_RGB;
+      bmi.bmiHeader.biSizeImage=0;
+      bmi.bmiHeader.biXPelsPerMeter=0;
+      bmi.bmiHeader.biYPelsPerMeter=0;
+      bmi.bmiHeader.biClrUsed=0;
+      bmi.bmiHeader.biClrImportant=0;
+
+      int bytes_per_line = (iw+1)*3 & ~3;
+      unsigned char* buf1 = new unsigned char[bytes_per_line * ih];
+      unsigned char* buf2 = new unsigned char[bytes_per_line * ih];
+      unsigned char* buf3 = new unsigned char[bytes_per_line * ih];
+      GetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+      GetDIBits(mdc2,pix2,0,ih,(void*)buf2,&bmi,DIB_RGB_COLORS);
+      GetDIBits(mdc,pix,0,ih,(void*)buf3,&bmi,DIB_RGB_COLORS);
+
+      int i,j;
+      for(i=0; i<ih; i++){
+        long ptr = i*bytes_per_line;
+        for(j=0; j<bytes_per_line; j++){
+          long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+          buf1[ptr] = val;
+          ptr++;
+        }
+      }
+
+      SetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+      BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x +sx,dy -y +sy,SRCCOPY);
+
+      SelectObject(mdc3,pxbk3);
+      DeleteObject(pix3);
+      DeleteDC(mdc3);
+      delete buf1;
+      delete buf2;
+      delete buf3;
+
+
     }
   }
 //printf("windraw::drawImage sx,sy=%d,%d dx,dy=%d,%d\n",sx,sy,dx,dy);
