@@ -38,7 +38,7 @@ WSDsocket* _winsocket_create_handler(){
   return sock;
 };
 
-static WSCbool initialized = False;
+WSCbool wsock_initialized = False;
 
 class  _winsocket_init {
   public:
@@ -47,9 +47,9 @@ class  _winsocket_init {
 //printf("WSDwinsocket init\n");
   };
   ~_winsocket_init(){
-     if (initialized != False){
+     if (wsock_initialized != False){
        WSACleanup();
-       initialized = False;
+       wsock_initialized = False;
      }
   };
 };
@@ -396,8 +396,8 @@ long WSDwinsocket::close(void* sock){
 }
 long WSDwinsocket::initialize(){
   if (_socket == 0){
-    if (initialized == False){
-      initialized = True;
+    if (wsock_initialized == False){
+      wsock_initialized = True;
       WORD wVersionReq;
       WSADATA wsaData;
       wVersionReq = MAKEWORD(2,0);
