@@ -30,10 +30,20 @@
 
 void _wsbuilder_lang_init_proc(){
 #include "English.cpp"
+}
+void _wsbuilder_lang_init_proc1(){
 #include "Japanese.cpp"
+}
+void _wsbuilder_lang_init_proc2(){
 #include "Japanese-sjis.cpp"
+}
+void _wsbuilder_lang_init_proc3(){
 #include "Japanese-utf8.cpp"
+}
+void _wsbuilder_lang_init_proc4(){
 #include "Korean.cpp"
+}
+void _wsbuilder_lang_init_proc5(){
 #include "Chinese-zh_cn.cpp"
 }
 
@@ -41,6 +51,11 @@ class _wsbuilder_lang_init_{
 public:
   _wsbuilder_lang_init_(){
     WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc);
+    WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc1);
+    WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc2);
+    WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc3);
+    WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc4);
+    WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc5);
   };
 };
 
