@@ -88,11 +88,11 @@ _xappDev_init _xappDev_init_run_constructor;
 void WSGFdeviceInitialize(){
   static WSCbool initialized = False;
   if (initialized == False){
-extern WSDfile* _unix_sem_chandler();
+extern WSDsemaphore* _unix_sem_chandler();
     WSDmutex::setCreateInstanceHandler((void*)_unix_sem_chandler);
-extern WSDfile* _unix_mutex_chandler();
+extern WSDmutex* _unix_mutex_chandler();
     WSDmutex::setCreateInstanceHandler((void*)_unix_mutex_chandler);
-extern WSDfile* _unix_th_chandler();
+extern WSDthread* _unix_th_chandler();
     WSDthread::setCreateInstanceHandler((void*)_unix_th_chandler);
 extern WSDfile* _unix_fp_chandler();
     WSDfile::setCreateInstanceHandler((void*)_unix_fp_chandler);
