@@ -662,7 +662,7 @@ long WSDxdraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* image
     xoff = ((short)w - (short)pw);
     yoff = ((short)h - (short)ph);
   }
-  if ((long)pix1 == -1 && pix1 != 0){
+  if ((long)pix1 == -1 || pix1 == 0){
     return WS_NO_ERR;
   }
   if ((long)pix2 != -1 && pix2 != 0){
@@ -922,7 +922,7 @@ long WSDxdraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCushort
   Pixmap pix1 = (Pixmap)image->getValue1();
   Pixmap pix2 = (Pixmap)image->getValue2();
 
-  if ((long)pix1 == -1 && pix1 != 0){
+  if ((long)pix1 == -1 || pix1 == 0){
     return WS_NO_ERR;
   }
   if ((long)pix2 != -1 && pix2 != 0){
@@ -962,7 +962,7 @@ long WSDxdraw::drawString(long x,long y,WSCulong w,WSCulong h,
          WSCstring* str,char font,char align,long cur,WSCbool inter_cur,
          long scur1,long scur2){
 #ifdef TDBG
-printf("WSDxdraw::drawString start %d\n",WSGFclocktime());
+  printf("WSDxdraw::drawString start %d\n",WSGFclocktime());
 #endif
 
   if (scur1 == scur2){
@@ -1105,7 +1105,7 @@ printf("WSDxdraw::drawString start %d\n",WSGFclocktime());
   setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
 
 #ifdef TDBG
-printf("WSDxdraw::drawString done %d\n",WSGFclocktime());
+  printf("WSDxdraw::drawString done %d\n",WSGFclocktime());
 #endif
 
   return WS_NO_ERR;
