@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -346,7 +346,7 @@ long WSDxfontSelect::initialize(){
 //  }
 
 
-  WSCvlabel* xfonvtex014 = new  WSCvlabel(xfonarea001,"xfonvtex014");
+  xfonvtex014 = new  WSCvlabel(xfonarea001,"xfonvtex014");
   xfonvtex014->initialize();
   xfonvtex014->setInternalObject(True);
   xfonvtex014->setUserData(WS_BASE_CLASS,(void*)this);
@@ -387,7 +387,7 @@ long WSDxfontSelect::initialize(){
   xfonvtex016->setPropertyV(WSNshadowType,(char)WS_SHADOW_TRANS);
   xfonvtex016->setVisible(True);
 #endif
-  WSCvlabel* xfonvtex017 = new  WSCvlabel(xfonarea001,"xfonvtex017");
+  xfonvtex017 = new  WSCvlabel(xfonarea001,"xfonvtex017");
   xfonvtex017->initialize();
   xfonvtex017->setInternalObject(True);
   xfonvtex017->setUserData(WS_BASE_CLASS,(void*)this);
@@ -628,6 +628,11 @@ void WSDxfontSelect::_add_name(char* data,WSClistData* list){
 
 void WSDxfontSelect::onVisibleChange(WSCbool fl){
   WSCbaseDialog::onVisibleChange(fl);
+  if (fl != False){
+    lb004->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR8));
+    xfonvtex014->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR1));
+    xfonvtex017->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR4));
+  }
   updateList();
 }
 
