@@ -266,8 +266,17 @@ void WSDxformDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,long dy
      XCopyArea( disp,pix,XtWindow(_widget),gc,
           (int)x, (int)y, (WSCuint)w, (WSCuint)h, (int)dx, (int)dy);
   }else{
-     XCopyArea( disp,XtWindow(_widget),XtWindow(_widget),gc,
+     if (gc != 0){
+       XCopyArea( disp,XtWindow(_widget),XtWindow(_widget),gc,
           (int)x, (int)y, (WSCuint)w, (WSCuint)h, (int)dx, (int)dy);
+     }
+     if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
+       Pixmap pix = getPixmap();
+       if (pix != 0 && gc != 0){
+         XCopyArea( disp,pix,pix,gc,
+            (int)x, (int)y, (WSCuint)w, (WSCuint)h, (int)dx, (int)dy);
+       }
+     }
      while(1){
        XFlush(disp);
        XEvent e;
