@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2003 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -40,6 +40,7 @@
 
 WSMFclassInit(WSDxkeyboard,WSDkeyboard);
 
+static WSCbool _has_no_focus_win = False;
 #ifndef KINPUT2
 #include <locale.h>
 #else
@@ -176,6 +177,10 @@ extern long WSGFgetLocaleValue(char*);
         NULL, NULL, NULL, _im_instantiate, (XPointer)this);
 #endif
 #endif
+  char* xmodifiers = getenv("XMODIFIERS");
+  if (xmodifiers != NULL && !strcmp(xmodifiers,"@im=Ami")){ //for ami
+    _has_no_focus_win = True;
+  }
   return WS_NO_ERR;
 }
 
@@ -183,7 +188,6 @@ long WSDxkeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgn
   WSDfont*    font  = WSGIappFontSet()->getFont(fontno);
   WSDcolor*   fgcol = WSGIappColorSet()->getColor(fgno);
   WSDcolor*   bgcol = WSGIappColorSet()->getColor(bgno);
-
 #ifdef KINPUT2
 
   setDevForKI(dev);
@@ -295,19 +299,24 @@ long WSDxkeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgn
            XNStatusDoneCallback,&callback_status_done, 
            XNStatusDrawCallback,&callback_status_draw, NULL,NULL);
 
-      Window win;
-      XGetICValues(_xic,XNFocusWindow,&win, NULL);
-      if(win != XtWindow((Widget)dev->getSpecialResource())){
-        XWindowAttributes att;
-        Display* disp =  XtDisplay((Widget)dev->getSpecialResource());
-        Window   target =  XtWindow((Widget)dev->getSpecialResource());
-        XGetWindowAttributes(disp, target, &att);
-        WSCulong mask;
-        XGetICValues(_xic,XNFilterEvents,&mask,NULL);
-        XSelectInput(disp,target,mask | att.your_event_mask);
-        _xic = _get_xic(target);
-        XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
+      if (_has_no_focus_win == False){
+        Window win;
+        XGetICValues(_xic,XNFocusWindow,&win, NULL);
+        if(win != XtWindow((Widget)dev->getSpecialResource())){
+          XWindowAttributes att;
+          Display* disp =  XtDisplay((Widget)dev->getSpecialResource());
+          Window   target =  XtWindow((Widget)dev->getSpecialResource());
+          XGetWindowAttributes(disp, target, &att);
+          WSCulong mask;
+          XGetICValues(_xic,XNFilterEvents,&mask,NULL);
+          XSelectInput(disp,target,mask | att.your_event_mask);
+          _xic = _get_xic(target);
+          XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
+                       list2, NULL);
+        }else{
+          XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
                      list2, NULL);
+        }
       }else{
         XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
                      list2, NULL);
@@ -532,7 +541,6 @@ static short y_bk = 0;
     area.y = 0;
     area.width =  10000;
     area.height = 10000;
-
     if (_xim_init != False){
       XVaNestedList list;
       XVaNestedList list2;
@@ -564,16 +572,36 @@ static short y_bk = 0;
       if (_xic == 0){
         _xic = _create_xic(XtWindow((Widget)dev->getSpecialResource()),
                          list,list2);
+        if (_xic != 0 && _has_no_focus_win == False){
+          Window win = 0;
+          XGetICValues(_xic,XNFocusWindow,&win, NULL);
+          if (win == 0){
+            _has_no_focus_win = True;
+          }
+        }
       }else{
-        Window win;
-        XGetICValues(_xic,XNFocusWindow,&win, NULL);
-        if(win != XtWindow((Widget)dev->getSpecialResource())){
+        Window win = 0;
+        if (_has_no_focus_win == False){ // for ami
+          XGetICValues(_xic,XNFocusWindow,&win, NULL);
+          if (win == 0){
+            _has_no_focus_win = True;
+          }
+        }
+        if(win == 0){
+          if (x_bk == x && y_bk == y){
+            point.y ++;
+            XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
+                         list2, NULL);
+            point.y --;
+          }
+          XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
+                       list2, NULL);
+       }else if(win != XtWindow((Widget)dev->getSpecialResource())){
           Display* disp =  XtDisplay((Widget)dev->getSpecialResource());
           Window   target =  XtWindow((Widget)dev->getSpecialResource());
           XWindowAttributes att;
           XGetWindowAttributes(disp, target, &att);
           XDestroyIC(_xic);
-
           _xic = NULL;
           _xic = _create_xic(XtWindow((Widget)dev->getSpecialResource()),
                            list,list2);
